@@ -1,4 +1,4 @@
-<?php // $Id: event.php,v 1.17.4.1 2004/06/03 08:14:06 moodler Exp $
+<?php // $Id: event.php,v 1.50.2.4 2006/07/17 07:29:46 skodak Exp $
 
 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
@@ -39,9 +39,9 @@
 /////////////////////////////////////////////////////////////////////////////
 
     require_once('../config.php');
-    require_once('lib.php');
-    require_once('../course/lib.php');
-    require_once('../mod/forum/lib.php');
+    require_once($CFG->dirroot.'/calendar/lib.php');
+    require_once($CFG->dirroot.'/course/lib.php');
+    require_once($CFG->dirroot.'/mod/forum/lib.php');
 
     require_login();
 
@@ -50,17 +50,22 @@
         redirect(CALENDAR_URL.'view.php?view=upcoming');
     }
 
-    require_variable($_REQUEST['action']);
-    optional_variable($_REQUEST['id']);
-    optional_variable($_REQUEST['type'], 'select');
-    $_REQUEST['id'] = intval($_REQUEST['id']); // Always a good idea, against SQL injections
+    $action = required_param('action', PARAM_ALPHA);
+    $eventid = optional_param('id', 0, PARAM_INT);
+    $eventtype = optional_param('type', 'select', PARAM_ALPHA);
+    $urlcourse = optional_param('course', 0, PARAM_INT);
+    $cal_y = optional_param('cal_y');
+    $cal_m = optional_param('cal_m');
+    $cal_d = optional_param('cal_d');
 
     if(!$site = get_site()) {
         redirect($CFG->wwwroot.'/'.$CFG->admin.'/index.php');
     }
 
+    $strcalendar = get_string('calendar', 'calendar');
+
     $now = usergetdate(time());
-    $nav = calendar_get_link_tag(get_string('calendar', 'calendar'), CALENDAR_URL.'view.php?view=upcoming&amp;', $now['mday'], $now['mon'], $now['year']);
+    $nav = calendar_get_link_tag($strcalendar, CALENDAR_URL.'view.php?view=upcoming&amp;', $now['mday'], $now['mon'], $now['year']);
     $day = intval($now['mday']);
     $mon = intval($now['mon']);
     $yr = intval($now['year']);
@@ -71,10 +76,24 @@
         $defaultformat = FORMAT_MOODLE;
     }
 
-    switch($_REQUEST['action']) {
+    // If a course has been supplied in the URL, change the filters to show that one
+    if($urlcourse > 0 && record_exists('course', 'id', $urlcourse)) {
+        if($urlcourse == SITEID) {
+            // If coming from the site page, show all courses
+            $SESSION->cal_courses_shown = calendar_get_default_courses(true);
+            calendar_set_referring_course(0);
+        }
+        else {
+            // Otherwise show just this one
+            $SESSION->cal_courses_shown = $urlcourse;
+            calendar_set_referring_course($SESSION->cal_courses_shown);
+        }
+    }
+
+    switch($action) {
         case 'delete':
             $title = get_string('deleteevent', 'calendar');
-            $event = get_record('event', 'id', $_REQUEST['id']);
+            $event = get_record('event', 'id', $eventid);
             if($event === false) {
                 error('Invalid event');
             }
@@ -85,7 +104,9 @@
 
         case 'edit':
             $title = get_string('editevent', 'calendar');
-            $event = get_record('event', 'id', $_REQUEST['id']);
+            $event = get_record('event', 'id', $eventid);
+            $repeats = optional_param('repeats', 0, PARAM_INT);
+
             if($event === false) {
                 error('Invalid event');
             }
@@ -95,8 +116,7 @@
 
             if($form = data_submitted()) {
 
-                $form->name = strip_tags($form->name);  // Strip all tags
-                $form->description = clean_text($form->description , $form->format);   // Clean up any bad tags
+                $form->name = clean_param(strip_tags($form->name,'<lang><span>'), PARAM_CLEAN);
 
                 $form->timestart = make_timestamp($form->startyr, $form->startmon, $form->startday, $form->starthr, $form->startmin);
                 if($form->duration == 1) {
@@ -106,18 +126,46 @@
                     }
                 }
                 else if($form->duration == 2) {
-                    $form->timeduration = $form->minutes * 60;
+                    $form->timeduration = $form->minutes * MINSECS;
                 }
                 else {
                     $form->timeduration = 0;
                 }
-                validate_form($form, $err);
-                if (count($err) == 0) {
-                    $form->timemodified = time();
-                    update_record('event', $form);
 
-                    /// Log the event update.
-                    add_to_log($form->courseid, 'calendar', 'edit', 'event.php?action=edit&amp;id='.$form->id, $form->name);
+                validate_form($form, $err);
+
+                if (count($err) == 0) {
+
+                    if($event->repeatid && $repeats) {
+                        // Update all
+                        if($form->timestart >= $event->timestart) {
+                            $timestartoffset = 'timestart + '.($form->timestart - $event->timestart);
+                        }
+                        else {
+                            $timestartoffset = 'timestart - '.($event->timestart - $form->timestart);
+                        }
+
+                        execute_sql('UPDATE '.$CFG->prefix.'event SET '.
+                            'name = '.$db->qstr($form->name).','.
+                            'description = '.$db->qstr($form->description).','.
+                            'timestart = '.$timestartoffset.','.
+                            'timeduration = '.$form->timeduration.','.
+                            'timemodified = '.time().' WHERE repeatid = '.$event->repeatid);
+                            
+                        /// Log the event update.
+                        $form->name = stripslashes($form->name);  //To avoid double-slashes
+                        add_to_log($form->courseid, 'calendar', 'edit all', 'event.php?action=edit&amp;id='.$form->id, $form->name);
+                    }
+
+                    else {
+                        // Update this
+                        $form->timemodified = time();
+                        update_record('event', $form);
+    
+                        /// Log the event update.
+                        $form->name = stripslashes($form->name);  //To avoid double-slashes
+                        add_to_log($form->courseid, 'calendar', 'edit', 'event.php?action=edit&amp;id='.$form->id, $form->name);
+                    }
 
                     // OK, now redirect to day view
                     redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
@@ -135,8 +183,7 @@
             $form = data_submitted();
             if(!empty($form) && $form->type == 'defined') {
 
-                $form->name = strip_tags($form->name);  // Strip all tags
-                $form->description = clean_text($form->description , $form->format);   // Clean up any bad tags
+                $form->name = clean_text(strip_tags($form->name, '<lang><span>'));
 
                 $form->timestart = make_timestamp($form->startyr, $form->startmon, $form->startday, $form->starthr, $form->startmin);
                 if($form->duration == 1) {
@@ -146,7 +193,7 @@
                     }
                 }
                 else if ($form->duration == 2) {
-                    $form->timeduration = $form->minutes * 60;
+                    $form->timeduration = $form->minutes * MINSECS;
                 }
                 else {
                     $form->timeduration = 0;
@@ -158,28 +205,40 @@
                 if (count($err) == 0) {
                     $form->timemodified = time();
 
+                    if ($form->repeat) {
+                        $fetch = get_record_sql('SELECT 1, MAX(repeatid) AS repeatid FROM '.$CFG->prefix.'event');
+                        $form->repeatid = empty($fetch) ? 1 : $fetch->repeatid + 1;
+                    }
+
                     /// Get the event id for the log record.
                     $eventid = insert_record('event', $form, true);
 
                     /// Log the event entry.
-                    add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, $form->name);
+                    add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, stripslashes($form->name));
 
                     if ($form->repeat) {
                         for($i = 1; $i < $form->repeats; $i++) {
-                            $form->timestart += 604800;  // add one week
+                            // What's the DST offset for the previous repeat?
+                            $dst_offset_prev = dst_offset_on($form->timestart);
+
+                            $form->timestart += WEEKSECS;
+
+                            // If the offset has changed in the meantime, update this repeat accordingly
+                            $form->timestart += $dst_offset_prev - dst_offset_on($form->timestart);
+
                             /// Get the event id for the log record.
                             $eventid = insert_record('event', $form, true);
+
                             /// Log the event entry.
-                            add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, $form->name);
+                            add_to_log($form->courseid, 'calendar', 'add', 'event.php?action=edit&amp;id='.$eventid, stripslashes($form->name));
                         }
                     }
-
                     // OK, now redirect to day view
                     redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$form->startday.'&cal_m='.$form->startmon.'&cal_y='.$form->startyr);
                 }
                 else {
                     foreach ($err as $key => $value) {
-                        $focus = 'form'.$key;
+                        $focus = 'form.'.$key;
                     }
                 }
             }
@@ -189,52 +248,73 @@
 
     // Let's see if we are supposed to provide a referring course link
     // but NOT for the "main page" course
-    if($SESSION->cal_course_referer > 1 &&
+    if($SESSION->cal_course_referer != SITEID &&
       ($shortname = get_field('course', 'shortname', 'id', $SESSION->cal_course_referer)) !== false) {
         // If we know about the referring course, show a return link
         $nav = '<a href="'.$CFG->wwwroot.'/course/view.php?id='.$SESSION->cal_course_referer.'">'.$shortname.'</a> -> '.$nav;
     }
 
-    print_header(get_string('calendar', 'calendar').': '.$title, $site->fullname, $nav.' -> '.$title,
-                 $focus, '', true, '', '<p class="logininfo">'.user_login_string($site).'</p>');
+    if (!empty($SESSION->cal_course_referer)) {
+        // TODO: This is part of the Great $course Hack in Moodle. Replace it at some point.
+        $course = get_record('course', 'id', $SESSION->cal_course_referer);
+    } else {
+        $course = $site;
+    }
+
+    print_header($site->shortname.': '.$strcalendar.': '.$title, $strcalendar, $nav.' -> '.$title,
+                 $focus, '', true, '', user_login_string($site));
 
     echo calendar_overlib_html();
 
-    echo '<table border="0" cellpadding="3" cellspacing="0" width="100%"><tr valign="top">';
-    echo '<td valign="top" width="100%">';
+    echo '<table id="calendar">';
+    echo '<tr><td class="maincalendar">';
 
-    switch($_REQUEST['action']) {
+    switch($action) {
         case 'delete':
-            if($_REQUEST['confirm'] == 1) {
+            $confirm = optional_param('confirm', 0, PARAM_INT);
+            $repeats = optional_param('repeats', 0, PARAM_INT);
+            if($confirm) {
                 // Kill it and redirect to day view
-                if(($event = get_record('event', 'id', $_REQUEST['id'])) !== false) {
-                    /// Log the event delete.
+                if(($event = get_record('event', 'id', $eventid)) !== false) {
 
-                    delete_records('event', 'id', $_REQUEST['id']);
-
-                    // pj - fixed the course id problem, but now we have another one:
-                    // what to do with the URL?
-                    add_to_log($event->courseid, 'calendar', 'delete', '', $event->name);
+                    if($event->repeatid && $repeats) {
+                        delete_records('event', 'repeatid', $event->repeatid);
+                        add_to_log($event->courseid, 'calendar', 'delete all', '', $event->name);
+                    }
+                    else {
+                        delete_records('event', 'id', $eventid);
+                        add_to_log($event->courseid, 'calendar', 'delete', '', $event->name);
+                    }
                 }
 
-                if(checkdate($_REQUEST['m'], $_REQUEST['d'], $_REQUEST['y'])) {
-                    // Being a bit paranoid to check this, but it doesn't hurt
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$_REQUEST['d'].'&cal_m='.$_REQUEST['m'].'&cal_y='.$_REQUEST['y']);
-                }
-                else {
-                    // Redirect to now
-                    redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$now['mday'].'&cal_m='.$now['mon'].'&cal_y='.$now['year']);
-                }
+                redirect(CALENDAR_URL.'view.php?view=day&cal_d='.$_REQUEST['d'].'&cal_m='.$_REQUEST['m'].'&cal_y='.$_REQUEST['y']);
+
             }
             else {
                 $eventtime = usergetdate($event->timestart);
                 $m = $eventtime['mon'];
                 $d = $eventtime['mday'];
                 $y = $eventtime['year'];
+
+                if($event->repeatid) {
+                    $fetch = get_record_sql('SELECT 1, COUNT(id) AS repeatcount FROM '.$CFG->prefix.'event WHERE repeatid = '.$event->repeatid);
+                    $repeatcount = $fetch->repeatcount;
+                }
+                else {
+                    $repeatcount = 0;
+                }
+                
                 // Display confirmation form
-                print_side_block_start(get_string('deleteevent', 'calendar').': '.$event->name, '', 'mycalendar');
+                echo '<div class="header">'.get_string('deleteevent', 'calendar').': '.$event->name.'</div>';
+                echo '<h2>'.get_string('confirmeventdelete', 'calendar').'</h2>';
+                if($repeatcount > 1) {
+                    echo '<p>'.get_string('youcandeleteallrepeats', 'calendar', $repeatcount).'</p>';
+                }
+                echo '<div class="eventlist">';
+                $event->time = calendar_format_event_time($event, time(), '', false);
+                calendar_print_event($event);
+                echo '</div>';
                 include('event_delete.html');
-                print_side_block_end();
             }
         break;
 
@@ -247,7 +327,7 @@
                 $form->timeduration = $event->timeduration;
                 $form->id = $event->id;
                 $form->format = $defaultformat;
-                if($event->timeduration > 3600) {
+                if($event->timeduration > HOURSECS) {
                     // More than one hour, so default to normal duration mode
                     $form->duration = 1;
                     $form->minutes = '';
@@ -255,7 +335,7 @@
                 else if($event->timeduration) {
                     // Up to one hour, "minutes" mode probably is better here
                     $form->duration = 2;
-                    $form->minutes = $event->timeduration / 60;
+                    $form->minutes = $event->timeduration / MINSECS;
                 }
                 else {
                     // No duration
@@ -263,29 +343,39 @@
                     $form->minutes = '';
                 }
             }
-            print_side_block_start(get_string('editevent', 'calendar'), '', 'mycalendar');
+
+            if (!empty($form->courseid)) {
+                // TODO: This is part of the Great $course Hack in Moodle. Replace it at some point.
+                $course = get_record('course', 'id', $form->courseid);
+            } else {
+                $course = $site;
+            }
+
+            if($event->repeatid) {
+                $fetch = get_record_sql('SELECT 1, COUNT(id) AS repeatcount FROM '.$CFG->prefix.'event WHERE repeatid = '.$event->repeatid);
+                $repeatcount = $fetch->repeatcount;
+            }
+            else {
+                $repeatcount = 0;
+            }
+
+            echo '<div class="header">'.get_string('editevent', 'calendar').'</div>';
             include('event_edit.html');
-            print_side_block_end();
             if ($usehtmleditor) {
                 use_html_editor("description");
             }
         break;
 
         case 'new':
-            optional_variable($_GET['cal_y']);
-            optional_variable($_GET['cal_m']);
-            optional_variable($_GET['cal_d']);
-            optional_variable($form->timestart, -1);
-
-            if($_GET['cal_y'] && $_GET['cal_m'] && $_GET['cal_d'] && checkdate($_GET['cal_m'], $_GET['cal_d'], $_GET['cal_y'])) {
-                $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], $_GET['cal_d'], 0, 0, 0);
+            if($cal_y && $cal_m && $cal_d && checkdate($cal_m, $cal_d, $cal_y)) {
+                $form->timestart = make_timestamp($cal_y, $cal_m, $cal_d, 0, 0, 0);
             }
-            else if($_GET['cal_y'] && $_GET['cal_m'] && checkdate($_GET['cal_m'], 1, $_GET['cal_y'])) {
-                if($_GET['cal_y'] == $now['year'] && $_GET['cal_m'] == $now['mon']) {
-                    $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], $now['mday'], 0, 0, 0);
+            else if($cal_y && $cal_m && checkdate($cal_m, 1, $cal_y)) {
+                if($cal_y == $now['year'] && $cal_m == $now['mon']) {
+                    $form->timestart = make_timestamp($cal_y, $cal_m, $now['mday'], 0, 0, 0);
                 }
                 else {
-                    $form->timestart = make_timestamp($_GET['cal_y'], $_GET['cal_m'], 1, 0, 0, 0);
+                    $form->timestart = make_timestamp($cal_y, $cal_m, 1, 0, 0, 0);
                 }
             }
             if($form->timestart < 0) {
@@ -295,12 +385,12 @@
             calendar_get_allowed_types($allowed);
             if(!$allowed->groups && !$allowed->courses && !$allowed->site) {
                 // Take the shortcut
-                $_REQUEST['type'] = 'user';
+                $eventtype = 'user';
             }
 
             $header = '';
 
-            switch($_REQUEST['type']) {
+            switch($eventtype) {
                 case 'user':
                     $form->name = '';
                     $form->description = '';
@@ -318,11 +408,10 @@
                     $header = get_string('typeuser', 'calendar');
                 break;
                 case 'group':
-                    optional_variable($_REQUEST['groupid']);
-                    $groupid = $_REQUEST['groupid'];
+                    $groupid = optional_param('groupid', 0, PARAM_INT);
                     if(!($group = get_record('groups', 'id', $groupid) )) {
                         calendar_get_allowed_types($allowed);
-                        $_REQUEST['type'] = 'select';
+                        $eventtype = 'select';
                     }
                     else {
                         $form->name = '';
@@ -342,11 +431,10 @@
                     }
                 break;
                 case 'course':
-                    optional_variable($_REQUEST['courseid']);
-                    $courseid = $_REQUEST['courseid'];
+                    $courseid = optional_param('courseid', 0, PARAM_INT);
                     if(!record_exists('course', 'id', $courseid)) {
                         calendar_get_allowed_types($allowed);
-                        $_REQUEST['type'] = 'select';
+                        $eventtype = 'select';
                     }
                     else {
                         $form->name = '';
@@ -368,7 +456,7 @@
                 case 'site':
                     $form->name = '';
                     $form->description = '';
-                    $form->courseid = 1;
+                    $form->courseid = SITEID;
                     $form->groupid = 0;
                     $form->userid = $USER->id;
                     $form->modulename = '';
@@ -393,20 +481,27 @@
                 $header = ' ('.$header.')';
             }
 
-            print_side_block_start(get_string('newevent', 'calendar').$header, '', 'mycalendar');
-            if($_REQUEST['type'] == 'select') {
-                $defaultcourse = $SESSION->cal_course_referer;
-                if(isteacheredit($defaultcourse, $USER->id)) {
-                    $defaultgroup = 0;
+            echo '<div class="header">'.get_string('newevent', 'calendar').$header.'</div>';
+
+            if($eventtype == 'select') {
+                $courseid = optional_param('courseid', $SESSION->cal_course_referer, PARAM_INT);
+                if ($courseid == 0) { // workaround by Dan for bug #6130
+                    $courseid = SITEID;
                 }
-                else {
-                    $defaultgroup = user_group($defaultcourse, $USER->id);
+                if (!$course = get_record('course', 'id', $courseid)) {
+                    error('Incorrect course ID');
                 }
-                optional_variable($_REQUEST['groupid'], $defaultgroup->id);
-                optional_variable($_REQUEST['courseid'], $defaultcourse);
-                $groupid = $_REQUEST['groupid'];
-                $courseid = $_REQUEST['courseid'];
+                if ($groupmode = groupmode($course)) {   // Groups are being used
+                    $changegroup = optional_param('group', -1, PARAM_INT);
+                    $groupid = get_and_set_current_group($course, $groupmode, $changegroup);
+                } else {
+                    $groupid = 0;
+                }
+
+                echo '<h2>'.get_string('eventkind', 'calendar').':</h2>';
+                echo '<div id="selecteventtype">';
                 include('event_select.html');
+                echo '</div>';
             }
             else {
                 include('event_new.html');
@@ -414,36 +509,36 @@
                     use_html_editor("description");
                 }
             }
-            print_side_block_end();
+
         break;
     }
     echo '</td>';
 
     // START: Last column (3-month display)
-    echo '<td style="vertical-align: top; width: 180px;">';
 
     $defaultcourses = calendar_get_default_courses();
     calendar_set_filters($courses, $groups, $users, $defaultcourses, $defaultcourses);
-
-    print_side_block_start(get_string('monthlyview', 'calendar'), '', 'sideblockmain');
     list($prevmon, $prevyr) = calendar_sub_month($mon, $yr);
     list($nextmon, $nextyr) = calendar_add_month($mon, $yr);
-
-    echo calendar_filter_controls('event', 'action='.$_REQUEST['action'].'&amp;type='.$_REQUEST['type'].'&amp;id='.$_REQUEST['id']);
-    echo '<p>';
+    
+    echo '<td class="sidecalendar">';
+    echo '<div class="header">'.get_string('monthlyview', 'calendar').'</div>';
+    echo '<div class="filters">';
+    echo calendar_filter_controls('event', 'action='.$action.'&amp;type='.$eventtype.'&amp;id='.$eventid);
+    echo '</div>';
+    
+    echo '<div class="minicalendarblock">';
     echo calendar_top_controls('display', array('m' => $prevmon, 'y' => $prevyr));
     echo calendar_get_mini($courses, $groups, $users, $prevmon, $prevyr);
-    echo '</p><p>';
+    echo '</div><div class="minicalendarblock">';
     echo calendar_top_controls('display', array('m' => $mon, 'y' => $yr));
     echo calendar_get_mini($courses, $groups, $users, $mon, $yr);
-    echo '</p><p>';
+    echo '</div><div class="minicalendarblock">';
     echo calendar_top_controls('display', array('m' => $nextmon, 'y' => $nextyr));
     echo calendar_get_mini($courses, $groups, $users, $nextmon, $nextyr);
-    echo '</p>';
-    print_side_block_end();
-    print_spacer(1, 180);
-    echo '</td>';
+    echo '</div>';
 
+    echo '</td>';
     echo '</tr></table>';
 
     print_footer();
@@ -493,7 +588,14 @@ function calendar_add_event_allowed($courseid, $groupid, $userid) {
     else if($courseid == 0 && $groupid == 0 && $userid == $USER->id) {
         return true;
     }
-    else if($courseid != 0 && isteacheredit($courseid)) {
+    else if($courseid == 0 && $groupid != 0) {
+        $group = get_record('groups', 'id', $groupid);
+        if($group === false) {
+            return false;
+        }
+        return isteacheredit($group->courseid) || isteacher($group->courseid) && ismember($groupid);
+    }
+    else if($courseid != 0 && isteacher($courseid)) {
         return true;
     }
 
@@ -506,24 +608,17 @@ function calendar_get_allowed_types(&$allowed) {
     $allowed->user = true; // User events always allowed
     $allowed->groups = false; // This may change just below
     $allowed->courses = false; // This may change just below
-    $allowed->site = isadmin($USER->id);
+    $allowed->site = isteacher(SITEID);
 
-    if(!empty($SESSION->cal_course_referer) && isteacheredit($SESSION->cal_course_referer, $USER->id)) {
-        $allowed->courses = array($SESSION->cal_course_referer => 1);
-        $allowed->groups = get_groups($SESSION->cal_course_referer);
-    }
+    if(!empty($SESSION->cal_course_referer) && $SESSION->cal_course_referer != SITEID && isteacher($SESSION->cal_course_referer, $USER->id)) {
+        $course = get_record('course', 'id', $SESSION->cal_course_referer);
 
-    //[pj]: This was used when we wanted to display all legal choices
-    /*
-    if($allowed->site) {
-        $allowed->courses = get_courses('all', 'c.shortname');
-        $allowed->groups = get_records_sql('SELECT g.*, c.fullname FROM '.$CFG->prefix.'groups g LEFT JOIN '.$CFG->prefix.'course c ON g.courseid = c.id ORDER BY c.shortname');
+        $allowed->courses = array($course->id => 1);
+
+        if($course->groupmode != NOGROUPS || !$course->groupmodeforce) {
+            $allowed->groups = get_groups($SESSION->cal_course_referer);
+        }
     }
-    else if(!empty($USER->teacheredit)) {
-        $allowed->courses = get_records_select('course', 'id != 1 AND id IN ('.implode(',', array_keys($USER->teacheredit)).')');
-        $allowed->groups = get_records_sql('SELECT g.*, c.fullname FROM '.$CFG->prefix.'groups g LEFT JOIN '.$CFG->prefix.'course c ON g.courseid = c.id WHERE g.courseid IN ('.implode(',', array_keys($USER->teacheredit)).')');
-    }
-    */
 }
 
 ?>
