@@ -1,4 +1,4 @@
-<?php // $Id: lib.php,v 1.125.2.3 2005/05/30 12:40:44 defacer Exp $
+<?php // $Id: lib.php,v 1.142.2.3 2006/07/24 00:14:07 danmarsden Exp $
 
 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
@@ -104,9 +104,15 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
     list($d, $m, $y) = array($date['mday'], $date['mon'], $date['year']); // This is what we want to display
     $display->maxdays = calendar_days_in_month($m, $y);
 
-    // We 'll keep these values as GMT here, and offset them when the time comes to query the db
-    $display->tstart = gmmktime(0, 0, 0, $m, 1, $y); // This is GMT
-    $display->tend = gmmktime(23, 59, 59, $m, $display->maxdays, $y); // GMT
+    if (get_user_timezone_offset() < 99) {
+        // We 'll keep these values as GMT here, and offset them when the time comes to query the db
+	    $display->tstart = gmmktime(0, 0, 0, $m, 1, $y); // This is GMT
+	    $display->tend = gmmktime(23, 59, 59, $m, $display->maxdays, $y); // GMT
+    } else {
+        // no timezone info specified
+	    $display->tstart = mktime(0, 0, 0, $m, 1, $y);
+	    $display->tend = mktime(23, 59, 59, $m, $display->maxdays, $y);
+    }
 
     $startwday = dayofweek(1, $m, $y);
 
@@ -116,6 +122,20 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
         $startwday += 7;
     }
 
+    // TODO: THIS IS TEMPORARY CODE!
+    // [pj] I was just reading through this and realized that I when writing this code I was probably
+    // asking for trouble, as all these time manipulations seem to be unnecessary and a simple
+    // make_timestamp would accomplish the same thing. So here goes a test:
+    //$test_start = make_timestamp($y, $m, 1);
+    //$test_end   = make_timestamp($y, $m, $display->maxdays, 23, 59, 59);
+    //if($test_start != usertime($display->tstart) - dst_offset_on($display->tstart)) {
+        //notify('Failed assertion in calendar/lib.php line 126; display->tstart = '.$display->tstart.', dst_offset = '.dst_offset_on($display->tstart).', usertime = '.usertime($display->tstart).', make_t = '.$test_start);
+    //}
+    //if($test_end != usertime($display->tend) - dst_offset_on($display->tend)) {
+        //notify('Failed assertion in calendar/lib.php line 130; display->tend = '.$display->tend.', dst_offset = '.dst_offset_on($display->tend).', usertime = '.usertime($display->tend).', make_t = '.$test_end);
+    //}
+
+
     // Get the events matching our criteria. Don't forget to offset the timestamps for the user's TZ!
     $whereclause = calendar_sql_where(
         usertime($display->tstart) - dst_offset_on($display->tstart),
@@ -147,7 +167,13 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
     //calendar_events_by_day($events, $display->tstart, $eventsbyday, $durationbyday, $typesbyday);
     calendar_events_by_day($events, $m, $y, $eventsbyday, $durationbyday, $typesbyday);
 
-    $content .= '<table class="minicalendar">'; // Begin table
+    //Accessibility: added summary and <abbr> elements.
+    ///global $CALENDARDAYS; appears to be broken.
+    $days_title = array('sunday','monday','tuesday','wednesday','thursday','friday','saturday');
+
+    $summary = get_string('calendarheading', 'calendar', userdate(make_timestamp($y, $m), get_string('strftimemonthyear')));
+    $summary = get_string('tabledata', 'access', $summary);
+    $content .= '<table class="minicalendar" summary="'.$summary.'">'; // Begin table
     $content .= '<tr class="weekdays">'; // Header row: day names
 
     // Print out the names of the weekdays
@@ -155,7 +181,8 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
     for($i = $display->minwday; $i <= $display->maxwday; ++$i) {
         // This uses the % operator to get the correct weekday no matter what shift we have
         // applied to the $display->minwday : $display->maxwday range from the default 0 : 6
-        $content .= '<th>'.get_string($days[$i % 7], 'calendar').'</th>';
+        $content .= '<th scope="col"><abbr title="'. get_string($days_title[$i % 7], 'calendar') .'">'.
+            get_string($days[$i % 7], 'calendar') ."</abbr></th>\n";
     }
 
     $content .= '</tr><tr>'; // End of day names; prepare for day numbers
@@ -168,8 +195,6 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
         $content .= '<td>&nbsp;</td>'."\n";
     }
 
-    $strftimetimedayshort = get_string('strftimedayshort');
-
     // Now display all the calendar
     for($day = 1; $day <= $display->maxdays; ++$day, ++$dayweek) {
         if($dayweek > $display->maxwday) {
@@ -219,10 +244,13 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
                 }
                 $popupcontent .= '<div><img height="16" width="16" src="'.$popupicon.'" style="vertical-align: middle; margin-right: 4px;" alt="'.$popupalt.'" /><a href="'.$dayhref.'">'.format_string($event->name,true).'</a></div>';
             }
-
-            $popupcaption = get_string('eventsfor', 'calendar', userdate($events[$eventid]->timestart, $strftimetimedayshort));
-            $popupcontent = str_replace("'", "\'", htmlspecialchars($popupcontent));
-            $popup = 'onmouseover="return overlib(\''.$popupcontent.'\', CAPTION, \''.$popupcaption.'\');" onmouseout="return nd();"';
+            
+            //Accessibility: functionality moved to calendar_get_popup.
+            if($display->thismonth && $day == $d) {
+                $popup = calendar_get_popup(true, $events[$eventid]->timestart, $popupcontent);
+            } else {
+                $popup = calendar_get_popup(false, $events[$eventid]->timestart, $popupcontent);
+            } 
 
             // Class and cell content
             if(isset($typesbyday[$day]['startglobal'])) {
@@ -257,8 +285,17 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
         }
 
         // Special visual fx for today
+        //Accessibility: hidden text for today, and popup.
         if($display->thismonth && $day == $d) {
             $class .= ' today';
+            $today = get_string('today', 'calendar').' '.userdate(time(), get_string('strftimedayshort'));
+                        
+            if(! isset($eventsbyday[$day])) {
+                $class .= ' eventnone';
+                $popup = calendar_get_popup(true, false);
+                $cell = '<a href="#" '.$popup.'>'.$day.'</a>';
+            }
+            $cell = '<span class="accesshide">'.$today.' </span>'.$cell;
         }
 
         // Just display it
@@ -279,6 +316,32 @@ function calendar_get_mini($courses, $groups, $users, $cal_month = false, $cal_y
     return $content;
 }
 
+/**
+ * calendar_get_popup, called at multiple points in from calendar_get_mini.
+ *        Copied and modified from calendar_get_mini.
+ * @uses OverLib popup.
+ * @param $is_today bool, false except when called on the current day.
+ * @param $event_timestart mixed, $events[$eventid]->timestart, OR false if there are no events.
+ * @param $popupcontent string.
+ * @return $popup string, contains onmousover and onmouseout events. 
+ */
+function calendar_get_popup($is_today, $event_timestart, $popupcontent='') {
+    $popupcaption = '';
+    if($is_today) {
+        $popupcaption = get_string('today', 'calendar').' ';
+    }
+    if (false === $event_timestart) {
+        $popupcaption .= userdate(time(), get_string('strftimedayshort'));
+        $popupcontent = get_string('eventnone', 'calendar');
+
+    } else {
+        $popupcaption .= get_string('eventsfor', 'calendar', userdate($event_timestart, get_string('strftimedayshort')));
+    }
+    $popupcontent = str_replace("'", "\'", htmlspecialchars($popupcontent));
+    $popup = 'onmouseover="return overlib(\''.$popupcontent.'\', CAPTION, \''.$popupcaption.'\');" onmouseout="return nd();"';
+    return $popup;
+}
+
 function calendar_get_upcoming($courses, $groups, $users, $daysinfuture, $maxevents, $fromtime=0) {
     global $CFG;
 
@@ -327,6 +390,7 @@ function calendar_get_upcoming($courses, $groups, $users, $daysinfuture, $maxeve
     }
 
     if($events !== false) {
+
         foreach($events as $event) {
 
             if($processed >= $display->maxevents) {
@@ -555,28 +619,32 @@ function calendar_top_controls($type, $data) {
     $data['m'] = $date['mon'];
     $data['y'] = $date['year'];
 
+    //Accessibility: calendar block controls, replaced <table> with <div>.
+    $nexttext = '<img src="'. $CFG->pixpath .'/a/r_next.gif" alt="'.get_string('monthnext','access').'" class="resize" />';
+    $prevtext = '<img src="'. $CFG->pixpath .'/a/r_previous.gif" alt="'.get_string('monthprev','access').'" class="resize" />';
+
     switch($type) {
         case 'frontpage':
             list($prevmonth, $prevyear) = calendar_sub_month($data['m'], $data['y']);
             list($nextmonth, $nextyear) = calendar_add_month($data['m'], $data['y']);
-            $nextlink = calendar_get_link_tag('&gt;&gt;', 'index.php?', 0, $nextmonth, $nextyear);
-            $prevlink = calendar_get_link_tag('&lt;&lt;', 'index.php?', 0, $prevmonth, $prevyear);
-            $content .= '<table class="calendar-controls"><tr>';
-            $content .= '<td class="previous">'.$prevlink."</td>\n";
-            $content .= '<td class="current"><a href="'.calendar_get_link_href(CALENDAR_URL.'view.php?view=month&amp;', 1, $data['m'], $data['y']).'">'.userdate($time, get_string('strftimemonthyear')).'</a></td>';
-            $content .= '<td class="next">'.$nextlink."</td>\n";
-            $content .= '</tr></table>';
+            $nextlink = calendar_get_link_tag($nexttext, 'index.php?', 0, $nextmonth, $nextyear);
+            $prevlink = calendar_get_link_tag($prevtext, 'index.php?', 0, $prevmonth, $prevyear);
+            $content .= '<div class="calendar-controls">';
+            $content .= '<span class="previous" title="'.get_string('monthprev','access').'">'.$prevlink."</span>\n";
+            $content .= '<span class="hide"> | </span><span class="current"><a href="'.calendar_get_link_href(CALENDAR_URL.'view.php?view=month&amp;', 1, $data['m'], $data['y']).'">'.userdate($time, get_string('strftimemonthyear')).'</a></span>';
+            $content .= '<span class="hide"> | </span><span class="next" title="'.get_string('monthnext','access').'">'.$nextlink."</span>\n";
+            $content .= "<span class=\"clearer\"></span></div>\n";
         break;
         case 'course':
             list($prevmonth, $prevyear) = calendar_sub_month($data['m'], $data['y']);
             list($nextmonth, $nextyear) = calendar_add_month($data['m'], $data['y']);
-            $nextlink = calendar_get_link_tag('&gt;&gt;', 'view.php?id='.$data['id'].'&amp;', 0, $nextmonth, $nextyear);
-            $prevlink = calendar_get_link_tag('&lt;&lt;', 'view.php?id='.$data['id'].'&amp;', 0, $prevmonth, $prevyear);
-            $content .= '<table class="calendar-controls"><tr>';
-            $content .= '<td class="previous">'.$prevlink."</td>\n";
-            $content .= '<td class="current"><a href="'.calendar_get_link_href(CALENDAR_URL.'view.php?view=month&amp;course='.$data['id'].'&amp;', 1, $data['m'], $data['y']).'">'.userdate($time, get_string('strftimemonthyear')).'</a></td>';
-            $content .= '<td class="next">'.$nextlink."</td>\n";
-            $content .= '</tr></table>';
+            $nextlink = calendar_get_link_tag($nexttext, 'view.php?id='.$data['id'].'&amp;', 0, $nextmonth, $nextyear);
+            $prevlink = calendar_get_link_tag($prevtext, 'view.php?id='.$data['id'].'&amp;', 0, $prevmonth, $prevyear);
+            $content .= '<div class="calendar-controls">';
+            $content .= '<span class="previous" title="'.get_string('monthprev','access').'">'.$prevlink."</span>\n";
+            $content .= '<span class="hide"> | </span><span class="current"><a href="'.calendar_get_link_href(CALENDAR_URL.'view.php?view=month&amp;course='.$data['id'].'&amp;', 1, $data['m'], $data['y']).'">'.userdate($time, get_string('strftimemonthyear')).'</a></span>';
+            $content .= '<span class="hide"> | </span><span class="next" title="'.get_string('monthnext','access').'">'.$nextlink."</span>\n";
+            $content .= "<span class=\"clearer\"></span></div>\n";
         break;
         case 'upcoming':
             $content .= '<div style="text-align: center;"><a href="'.CALENDAR_URL.'view.php?view=upcoming">'.userdate($time, get_string('strftimemonthyear'))."</a></div>\n";
@@ -589,11 +657,11 @@ function calendar_top_controls($type, $data) {
             list($nextmonth, $nextyear) = calendar_add_month($data['m'], $data['y']);
             $prevdate = make_timestamp($prevyear, $prevmonth, 1);
             $nextdate = make_timestamp($nextyear, $nextmonth, 1);
-            $content .= '<table class="calendar-controls"><tr>';
-            $content .= '<td class="previous"><a href="'.calendar_get_link_href('view.php?view=month&amp;', 1, $prevmonth, $prevyear).'">&lt;&lt; '.userdate($prevdate, get_string('strftimemonthyear')).'</a></td>';
-            $content .= '<td class="current">'.userdate($time, get_string('strftimemonthyear'))."</td>\n";
-            $content .= '<td class="next"><a href="'.calendar_get_link_href('view.php?view=month&amp;', 1, $nextmonth, $nextyear).'">'.userdate($nextdate, get_string('strftimemonthyear'))." &gt;&gt;</a></td>\n";
-            $content .= "</tr></table>\n";
+            $content .= '<div class="calendar-controls">';
+            $content .= '<span class="previous"><a href="'.calendar_get_link_href('view.php?view=month&amp;', 1, $prevmonth, $prevyear).'">&lt; '.userdate($prevdate, get_string('strftimemonthyear')).'</a></span>';
+            $content .= '<span class="hide"> | </span><span class="current">'.userdate($time, get_string('strftimemonthyear'))."</span>\n";
+            $content .= '<span class="hide"> | </span><span class="next"><a href="'.calendar_get_link_href('view.php?view=month&amp;', 1, $nextmonth, $nextyear).'">'.userdate($nextdate, get_string('strftimemonthyear'))." &gt;</a></span>\n";
+            $content .= "<span class=\"clearer\"></span></div>\n";
         break;
         case 'day':
             $data['d'] = $date['mday']; // Just for convenience
@@ -601,8 +669,8 @@ function calendar_top_controls($type, $data) {
             $nextdate = usergetdate(make_timestamp($data['y'], $data['m'], $data['d'] + 1));
             $prevname = calendar_wday_name($CALENDARDAYS[$prevdate['wday']]);
             $nextname = calendar_wday_name($CALENDARDAYS[$nextdate['wday']]);
-            $content .= '<table class="calendar-controls"><tr>';
-            $content .= '<td class="previous"><a href="'.calendar_get_link_href('view.php?view=day&amp;', $prevdate['mday'], $prevdate['mon'], $prevdate['year']).'">&lt;&lt; '.$prevname."</a></td>\n";
+            $content .= '<div class="calendar-controls">';
+            $content .= '<span class="previous"><a href="'.calendar_get_link_href('view.php?view=day&amp;', $prevdate['mday'], $prevdate['mon'], $prevdate['year']).'">&lt; '.$prevname."</a></span>\n";
 
             // Get the format string
             $text = get_string('strftimedaydate');
@@ -614,10 +682,10 @@ function calendar_top_controls($type, $data) {
             // Replace with actual values and lose any day leading zero
             $text = userdate($time, $text);
             // Print the actual thing
-            $content .= '<td class="current">'.$text.'</td>';
+            $content .= '<span class="hide"> | </span><span class="current">'.$text.'</span>';
 
-            $content .= '<td class="next"><a href="'.calendar_get_link_href('view.php?view=day&amp;', $nextdate['mday'], $nextdate['mon'], $nextdate['year']).'">'.$nextname." &gt;&gt;</a></td>\n";
-            $content .= '</tr></table>';
+            $content .= '<span class="hide"> | </span><span class="next"><a href="'.calendar_get_link_href('view.php?view=day&amp;', $nextdate['mday'], $nextdate['mon'], $nextdate['year']).'">'.$nextname." &gt;</a></span>\n";
+            $content .= "<span class=\"clearer\"></span></div>\n";
         break;
     }
     return $content;
@@ -628,6 +696,8 @@ function calendar_filter_controls($type, $vars = NULL, $course = NULL) {
 
     $groupevents = true;
     $getvars = '';
+   
+    $id = optional_param( 'id',0,PARAM_INT );
 
     switch($type) {
         case 'event':
@@ -637,8 +707,8 @@ function calendar_filter_controls($type, $vars = NULL, $course = NULL) {
             $getvars = '&amp;from='.$type;
         break;
         case 'course':
-            if (isset($_GET['id'])) {
-                $getvars = '&amp;from=course&amp;id='.$_GET['id'];
+            if ($id > 0) {
+                $getvars = '&amp;from=course&amp;id='.$id;
             } else {
                 $getvars = '&amp;from=course';
             }
@@ -790,7 +860,7 @@ function calendar_get_sideblock_upcoming($events, $linkhref = NULL) {
             continue;
         }
         $events[$i] = calendar_add_event_metadata($events[$i]);
-        $content .= '<div class="event">'.$events[$i]->icon.' ';
+        $content .= '<div class="event"><span class="icon c0">'.$events[$i]->icon.'</span> ';
         if (!empty($events[$i]->referer)) {
             // That's an activity event, so let's provide the hyperlink
             $content .= $events[$i]->referer;
@@ -1023,6 +1093,10 @@ function calendar_set_filters(&$courses, &$group, &$user, $courseeventsfrom = NU
     else {
         $courses = false;
     }
+   //BUG 6130 clean $courses array as SESSION has bad entries. 
+   foreach ($courses as $index => $value) {
+       if (empty($value)) unset($courses[$index]);
+   }
 
     if($SESSION->cal_show_user || $ignorefilters) {
         // This doesn't work for arrays yet (maybe someday it will)
@@ -1066,7 +1140,10 @@ function calendar_set_filters(&$courses, &$group, &$user, $courseeventsfrom = NU
 
                 // Otherwise (not editing teacher) show events from the group he is a member of
                 else if(isset($USER->groupmember[$courseid])) {
-                    $grouparray[] = $USER->groupmember[$courseid];
+                    //changed to 2D array
+                    foreach ($USER->groupmember[$courseid] as $groupid){
+                        $grouparray[] = $groupid;
+                    }
                 }
             }
             if(empty($grouparray)) {
@@ -1128,10 +1205,10 @@ function calendar_get_default_courses($ignoreref = false) {
         }
     }
     if(isset($USER->student) && is_array($USER->student)) {
-        $courses = $USER->student;
+        $courses = $courses + $USER->student;
     }
     if(isset($USER->teacher) && is_array($USER->teacher)) {
-        $courses = $USER->teacher;
+        $courses = $courses + $USER->teacher;
     }
     return $courses;
 }
