@@ -1,4 +1,4 @@
-<?PHP // $Id: unenrol.php,v 1.14.4.1 2004/10/09 19:06:51 stronk7 Exp $
+<?php // $Id: unenrol.php,v 1.21 2006/04/12 16:59:51 skodak Exp $
 
 //  Removes a student from a class
 //  This will not delete any of their data from the course, 
@@ -8,8 +8,9 @@
     require_once("../config.php");
     require_once("lib.php");
 
-    require_variable($id);               //course
-    optional_variable($user, $USER->id); //user
+    $id      = required_param('id', PARAM_INT);               //course
+    $user    = optional_param('user', $USER->id, PARAM_INT);  //user
+    $confirm = optional_param('confirm', 0, PARAM_BOOL);
 
     if (! $course = get_record("course", "id", $id) ) {
         error("That's an invalid course id");
@@ -24,11 +25,11 @@
         error("You must be a teacher with editing rights to do this");
     }
 
-    if ($user->id == $USER->id and !$CFG->allowunenroll) {
+    if ($user->id == $USER->id and !$CFG->allowunenroll or $course->metacourse) {
         error("You are not allowed to unenroll");
     }
 
-    if (isset($confirm) and confirm_sesskey()) {
+    if ($confirm and confirm_sesskey()) {
 
         if (! unenrol_student($user->id, $course->id)) {
             error("An error occurred while trying to unenrol you.");
@@ -48,7 +49,7 @@
     $strunenrol = get_string("unenrol");
 
     print_header("$course->shortname: $strunenrol", "$course->fullname", 
-                 "<A HREF=\"$CFG->wwwroot/course/view.php?id=$course->id\">$course->shortname</A> -> $strunenrol"); 
+                 "<a href=\"$CFG->wwwroot/course/view.php?id=$course->id\">$course->shortname</a> -> $strunenrol"); 
 
     if ($user->id == $USER->id) {
         $strunenrolsure  = get_string("unenrolsure", "", get_string("yourself"));
@@ -56,8 +57,8 @@
         $strunenrolsure = get_string("unenrolsure", "", fullname($user, true));
     }
 
-    notice_yesno ($strunenrolsure, "unenrol.php?id=$id&user=$user->id&confirm=yes&sesskey=$USER->sesskey", "$HTTP_REFERER");
+    notice_yesno ($strunenrolsure, "unenrol.php?id=$id&amp;user=$user->id&amp;confirm=yes&amp;sesskey=$USER->sesskey", $_SERVER['HTTP_REFERER']);
 
-    print_footer();
+    print_footer($course);
 
 ?>
