@@ -1,4 +1,4 @@
-<?php // $Id: index.php,v 1.72.2.5 2006/01/06 05:14:27 moodler Exp $
+<?php // $Id: index.php,v 1.86.2.7 2006/08/30 08:04:43 skodak Exp $
 
 //  Manage all uploaded files in a course file area
 
@@ -16,8 +16,11 @@
     $action  = optional_param('action', '', PARAM_ACTION);
     $name    = optional_param('name', '', PARAM_FILE);
     $oldname = optional_param('oldname', '', PARAM_FILE);
-    $choose  = optional_param('choose', '', PARAM_CLEAN);
-
+    $choose  = optional_param('choose', '', PARAM_FILE); //in fact it is always 'formname.inputname'
+    $userfile= optional_param('userfile','',PARAM_FILE);
+    $save    = optional_param('save', 0, PARAM_BOOL);
+    $text    = optional_param('text', '', PARAM_RAW);
+    $confirm = optional_param('confirm', 0, PARAM_BOOL);
 
     if ($choose) {
         if (count(explode('.', $choose)) != 2) {
@@ -37,17 +40,13 @@
     }
 
     function html_footer() {
-
         global $course, $choose;
 
-        if ($choose) {
-            echo "</td></tr></table></body></html>";
-        } else {
-            echo "</td></tr></table></body></html>";
-            print_footer($course);
-        }
+        echo '</td></tr></table>';
+
+        print_footer($course);
     }
-    
+
     function html_header($course, $wdir, $formfield=""){
         global $CFG, $ME, $choose;
 
@@ -133,15 +132,12 @@
 //  End of configuration and access control
 
 
-    if (!$wdir) {
-        $wdir="/";
+    if ($wdir == '') {
+        $wdir = "/";
     }
 
-    if (($wdir != '/' and detect_munged_arguments($wdir, 0))
-      or ($file != '' and detect_munged_arguments($file, 0))) {
-        $message = "Error: Directories can not contain \"..\"";
-        $wdir = "/";
-        $action = "";
+    if ($wdir{0} != '/') {  //make sure $wdir starts with slash
+        $wdir = "/".$wdir;
     }
 
     if ($wdir == "/backupdata") {
@@ -150,14 +146,18 @@
         }
     }
 
+    if (!is_dir($basedir.$wdir)) {
+        html_header($course, $wdir);
+        error("Requested directory does not exist.", "$CFG->wwwroot/files/index.php?id=$id");
+    }
 
     switch ($action) {
 
         case "upload":
             html_header($course, $wdir);
             require_once($CFG->dirroot.'/lib/uploadlib.php');
-                
-            if (!empty($save) and confirm_sesskey()) {
+
+            if ($save and confirm_sesskey()) {
                 $course->maxbytes = 0;  // We are ignoring course limits
                 $um = new upload_manager('userfile',false,false,$course,false,0);
                 $dir = "$basedir$wdir";
@@ -175,21 +175,21 @@
                 $strmaxsize = get_string("maxsize", "", $filesize);
                 $strcancel = get_string("cancel");
 
-                echo "<p>$struploadafile ($strmaxsize) --> <b>$wdir</b>";
+                echo "<p>$struploadafile ($strmaxsize) --> <b>$wdir</b></p>";
                 echo "<table><tr><td colspan=\"2\">";
                 echo "<form enctype=\"multipart/form-data\" method=\"post\" action=\"index.php\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"upload\" />";
                 echo " <input type=\"hidden\" name=\"sesskey\" value=\"$USER->sesskey\" />";
                 upload_print_form_fragment(1,array('userfile'),null,false,null,$upload_max_filesize,0,false);
-                echo " </td><tr><td width=\"10\">";
+                echo " </td></tr><tr><td width=\"10\">";
                 echo " <input type=\"submit\" name=\"save\" value=\"$struploadthisfile\" />";
                 echo "</form>";
                 echo "</td><td width=\"100%\">";
                 echo "<form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -201,12 +201,14 @@
             break;
 
         case "delete":
-            if (!empty($confirm) and confirm_sesskey()) {
+            if ($confirm and confirm_sesskey()) {
                 html_header($course, $wdir);
-                foreach ($USER->filelist as $file) {
-                    $fullfile = $basedir.$file;
-                    if (! fulldelete($fullfile)) {
-                        echo "<br />Error: Could not delete: $fullfile";
+                if (!empty($USER->filelist)) {
+                    foreach ($USER->filelist as $file) {
+                        $fullfile = $basedir.'/'.$file;
+                        if (! fulldelete($fullfile)) {
+                            echo "<br />Error: Could not delete: $fullfile";
+                        }
                     }
                 }
                 clearfilelist();
@@ -215,13 +217,33 @@
 
             } else {
                 html_header($course, $wdir);
+
                 if (setfilelist($_POST)) {
-                    echo "<p align=\"center\">".get_string("deletecheckwarning").":</p>";
+                    notify(get_string('deletecheckwarning').':');
                     print_simple_box_start("center");
                     printfilelist($USER->filelist);
                     print_simple_box_end();
                     echo "<br />";
-                    notice_yesno (get_string("deletecheckfiles"), 
+
+                    $resourcelist = false;
+                    foreach ($USER->filelist as $file) {
+                        // If file is specified in a resource, then delete that too.
+                        $clean_name = substr($file, 1);
+                        if (record_exists('resource', 'reference', $clean_name)) {
+                            if (!$resourcelist) {
+                                print_simple_box_start("center");
+                                $resourcelist = true;
+                            }
+                            $resource_id = files_get_cm_from_resource_name($clean_name);
+                            echo '<p>'.get_string('warningdeleteresource', '', $file)." <a href='$CFG->wwwroot/course/mod.php?update=$resource_id&sesskey=$USER->sesskey'>".get_string('update')."</a></p>";
+                        }
+                    }
+                    if ($resourcelist) {
+                        print_simple_box_end();
+                        echo "<br />";
+                    }
+
+                    notice_yesno (get_string("deletecheckfiles"),
                                 "index.php?id=$id&amp;wdir=$wdir&amp;action=delete&amp;confirm=1&amp;sesskey=$USER->sesskey&amp;choose=$choose",
                                 "index.php?id=$id&amp;wdir=$wdir&amp;action=cancel&amp;choose=$choose");
                 } else {
@@ -249,7 +271,7 @@
             if (isset($USER->fileop) and ($USER->fileop == "move") and confirm_sesskey()) {
                 foreach ($USER->filelist as $file) {
                     $shortfile = basename($file);
-                    $oldfile = $basedir.$file;
+                    $oldfile = $basedir.'/'.$file;
                     $newfile = $basedir.$wdir."/".$shortfile;
                     if (!rename($oldfile, $newfile)) {
                         echo "<p>Error: $shortfile not moved";
@@ -262,16 +284,33 @@
             break;
 
         case "rename":
-            if (!empty($name) and confirm_sesskey()) {
+            if (($name != '') and confirm_sesskey()) {
                 html_header($course, $wdir);
                 $name = clean_filename($name);
                 if (file_exists($basedir.$wdir."/".$name)) {
-                    echo "Error: $name already exists!";
+                    echo "<center>Error: $name already exists!</center>";
                 } else if (!rename($basedir.$wdir."/".$oldname, $basedir.$wdir."/".$name)) {
-                    echo "Error: could not rename $oldname to $name";
+                    echo "<p align=\"center\">Error: could not rename $oldname to $name</p>";
+                }
+
+                //if file is part of resource then update resource table as well
+                //this line only catch the root directory
+                if (record_exists('resource', 'reference', $oldname)) {
+                    set_field('resource', 'reference', $name, 'reference', $oldname);
+                }
+
+                if (get_dir_name_from_resource($oldname)) {
+                    $resources = get_dir_name_from_resource($oldname);
+                    print_simple_box_start("center");
+                    echo "<b>The following files might be referenced as a resource :</b><br>";
+                    foreach ($resources as $resource) {
+                        $resource_id = files_get_cm_from_resource_name($name);
+                        echo '<p align=\"center\">'. "$resource->reference :"."</align><a href='$CFG->wwwroot/course/mod.php?update=$resource_id&sesskey=$USER->sesskey'> ".get_string('update')."</a>";
+                    }
+                    print_simple_box_end();
                 }
                 displaydir($wdir);
-                    
+
             } else {
                 $strrename = get_string("rename");
                 $strcancel = get_string("cancel");
@@ -280,7 +319,7 @@
                 echo "<p>$strrenamefileto:";
                 echo "<table><tr><td>";
                 echo "<form action=\"index.php\" method=\"post\" name=\"form\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"rename\" />";
@@ -291,7 +330,7 @@
                 echo "</form>";
                 echo "</td><td>";
                 echo "<form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -303,25 +342,25 @@
             break;
 
         case "makedir":
-            if (!empty($name) and confirm_sesskey()) {
+            if (($name != '') and confirm_sesskey()) {
                 html_header($course, $wdir);
                 $name = clean_filename($name);
                 if (file_exists("$basedir$wdir/$name")) {
                     echo "Error: $name already exists!";
-                } else if (! make_upload_directory("$course->id/$wdir/$name")) {
+                } else if (! make_upload_directory("$course->id$wdir/$name")) {
                     echo "Error: could not create $name";
                 }
                 displaydir($wdir);
-                    
+
             } else {
                 $strcreate = get_string("create");
                 $strcancel = get_string("cancel");
                 $strcreatefolder = get_string("createfolder", "moodle", $wdir);
                 html_header($course, $wdir, "form.name");
-                echo "<p>$strcreatefolder:";
+                echo "<p>$strcreatefolder:</p>";
                 echo "<table><tr><td>";
                 echo "<form action=\"index.php\" method=\"post\" name=\"form\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"makedir\" />";
@@ -331,7 +370,7 @@
                 echo "</form>";
                 echo "</td><td>";
                 echo "<form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -344,16 +383,16 @@
 
         case "edit":
             html_header($course, $wdir);
-            if (isset($text) and confirm_sesskey()) {
-                $fileptr = fopen($basedir.$file,"w");
+            if (($text != '') and confirm_sesskey()) {
+                $fileptr = fopen($basedir.'/'.$file,"w");
                 fputs($fileptr, stripslashes($text));
                 fclose($fileptr);
                 displaydir($wdir);
-                    
+
             } else {
                 $streditfile = get_string("edit", "", "<b>$file</b>");
-                $fileptr  = fopen($basedir.$file, "r");
-                $contents = fread($fileptr, filesize($basedir.$file));
+                $fileptr  = fopen($basedir.'/'.$file, "r");
+                $contents = fread($fileptr, filesize($basedir.'/'.$file));
                 fclose($fileptr);
 
                 if (mimeinfo("type", $file) == "text/html") {
@@ -367,7 +406,7 @@
 
                 echo "<table><tr><td colspan=\"2\">";
                 echo "<form action=\"index.php\" method=\"post\" name=\"form\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"file\" value=\"$file\" />";
@@ -379,7 +418,7 @@
                 echo "</form>";
                 echo "</td><td>";
                 echo "<form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -387,7 +426,7 @@
                 echo "</form>";
                 echo "</td></tr></table>";
 
-                if ($usehtmleditor) { 
+                if ($usehtmleditor) {
                     use_html_editor();
                 }
 
@@ -397,7 +436,7 @@
             break;
 
         case "zip":
-            if (!empty($name) and confirm_sesskey()) {
+            if (($name != '') and confirm_sesskey()) {
                 html_header($course, $wdir);
                 $name = clean_filename($name);
 
@@ -406,13 +445,13 @@
                    $files[] = "$basedir/$file";
                 }
 
-                if (!zip_files($files,"$basedir/$wdir/$name")) {
+                if (!zip_files($files,"$basedir$wdir/$name")) {
                     error(get_string("zipfileserror","error"));
                 }
 
                 clearfilelist();
                 displaydir($wdir);
-                    
+
             } else {
                 html_header($course, $wdir, "form.name");
 
@@ -422,10 +461,10 @@
                     printfilelist($USER->filelist);
                     print_simple_box_end();
                     echo "<br />";
-                    echo "<p align=\"center\">".get_string("whattocallzip");
+                    echo "<p align=\"center\">".get_string("whattocallzip")."</p>";
                     echo "<table><tr><td>";
                     echo "<form action=\"index.php\" method=\"post\" name=\"form\">";
-                    echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                    echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                     echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                     echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                     echo " <input type=\"hidden\" name=\"action\" value=\"zip\" />";
@@ -435,7 +474,7 @@
                     echo "</form>";
                     echo "</td><td>";
                     echo "<form action=\"index.php\" method=\"get\">";
-                    echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                    echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                     echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                     echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                     echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -452,7 +491,7 @@
 
         case "unzip":
             html_header($course, $wdir);
-            if (!empty($file) and confirm_sesskey()) {
+            if (($file != '') and confirm_sesskey()) {
                 $strok = get_string("ok");
                 $strunpacking = get_string("unpacking", "", $file);
 
@@ -460,12 +499,12 @@
 
                 $file = basename($file);
 
-                if (!unzip_file("$basedir/$wdir/$file")) {
+                if (!unzip_file("$basedir$wdir/$file")) {
                     error(get_string("unzipfileserror","error"));
                 }
 
                 echo "<center><form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -480,7 +519,7 @@
 
         case "listzip":
             html_header($course, $wdir);
-            if (!empty($file) and confirm_sesskey()) {
+            if (($file != '') and confirm_sesskey()) {
                 $strname = get_string("name");
                 $strsize = get_string("size");
                 $strmodified = get_string("modified");
@@ -491,8 +530,8 @@
                 $file = basename($file);
 
                 include_once("$CFG->libdir/pclzip/pclzip.lib.php");
-                $archive = new PclZip(cleardoubleslashes("$basedir/$wdir/$file"));
-                if (!$list = $archive->listContent(cleardoubleslashes("$basedir/$wdir"))) {
+                $archive = new PclZip(cleardoubleslashes("$basedir$wdir/$file"));
+                if (!$list = $archive->listContent(cleardoubleslashes("$basedir$wdir"))) {
                     notify($archive->errorInfo(true));
 
                 } else {
@@ -500,7 +539,7 @@
                     echo "<tr class=\"file\"><th align=\"left\" class=\"header name\">$strname</th><th align=\"right\" class=\"header size\">$strsize</th><th align=\"right\" class=\"header date\">$strmodified</th></tr>";
                     foreach ($list as $item) {
                         echo "<tr>";
-                        print_cell("left", $item['filename'], 'name');
+                        print_cell("left", s($item['filename']), 'name');
                         if (! $item['folder']) {
                             print_cell("right", display_size($item['size']), 'size');
                         } else {
@@ -513,7 +552,7 @@
                     echo "</table>";
                 }
                 echo "<br /><center><form action=\"index.php\" method=\"get\">";
-                echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+                echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
                 echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
                 echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
                 echo " <input type=\"hidden\" name=\"action\" value=\"cancel\" />";
@@ -528,7 +567,7 @@
 
         case "restore":
             html_header($course, $wdir);
-            if (!empty($file) and confirm_sesskey()) {
+            if (($file != '') and confirm_sesskey()) {
                 echo "<p align=\"center\">".get_string("youaregoingtorestorefrom").":</p>";
                 print_simple_box_start("center");
                 echo $file;
@@ -537,14 +576,14 @@
                 echo "<p align=\"center\">".get_string("areyousuretorestorethisinfo")."</p>";
                 $restore_path = "$CFG->wwwroot/backup/restore.php";
                 notice_yesno (get_string("areyousuretorestorethis"),
-                                $restore_path."?id=".$id."&amp;file=".cleardoubleslashes($id.$wdir."/".$file),
+                                $restore_path."?id=".$id."&amp;file=".cleardoubleslashes($id.$wdir."/".$file)."&amp;method=manual",
                                 "index.php?id=$id&amp;wdir=$wdir&amp;action=cancel");
             } else {
                 displaydir($wdir);
             }
             html_footer();
             break;
-          
+
         case "cancel":
             clearfilelist();
 
@@ -570,9 +609,7 @@ function setfilelist($VARS) {
         if (substr($key,0,4) == "file") {
             $count++;
             $val = rawurldecode($val);
-            if (!detect_munged_arguments($val, 0)) {
-                $USER->filelist[] = $val;
-            }
+            $USER->filelist[] = clean_param($val, PARAM_PATH);
         }
     }
     return $count;
@@ -590,10 +627,10 @@ function printfilelist($filelist) {
     global $CFG, $basedir;
 
     foreach ($filelist as $file) {
-        if (is_dir($basedir.$file)) {
+        if (is_dir($basedir.'/'.$file)) {
             echo "<img src=\"$CFG->pixpath/f/folder.gif\" height=\"16\" width=\"16\" alt=\"\" /> $file<br />";
             $subfilelist = array();
-            $currdir = opendir($basedir.$file);
+            $currdir = opendir($basedir.'/'.$file);
             while (false !== ($subfile = readdir($currdir))) {
                 if ($subfile <> ".." && $subfile <> ".") {
                     $subfilelist[] = $file."/".$subfile;
@@ -601,7 +638,7 @@ function printfilelist($filelist) {
             }
             printfilelist($subfilelist);
 
-        } else { 
+        } else {
             $icon = mimeinfo("icon", $file);
             echo "<img src=\"$CFG->pixpath/f/$icon\"  height=\"16\" width=\"16\" alt=\"\" /> $file<br />";
         }
@@ -625,13 +662,14 @@ function displaydir ($wdir) {
     global $choose;
 
     $fullpath = $basedir.$wdir;
+    $dirlist = array();
 
     $directory = opendir($fullpath);             // Find all files
     while (false !== ($file = readdir($directory))) {
         if ($file == "." || $file == "..") {
             continue;
         }
-        
+
         if (is_dir($fullpath."/".$file)) {
             $dirlist[] = $file;
         } else {
@@ -646,6 +684,8 @@ function displaydir ($wdir) {
     $straction = get_string("action");
     $strmakeafolder = get_string("makeafolder");
     $struploadafile = get_string("uploadafile");
+    $strselectall = get_string("selectall");
+    $strselectnone = get_string("deselectall");
     $strwithchosenfiles = get_string("withchosenfiles");
     $strmovetoanotherfolder = get_string("movetoanotherfolder");
     $strmovefilestohere = get_string("movefilestohere");
@@ -660,9 +700,9 @@ function displaydir ($wdir) {
 
 
     echo "<form action=\"index.php\" method=\"post\" name=\"dirform\">";
-    echo '<input type="hidden" name="choose" value="'.$choose.'">';
+    echo '<input type="hidden" name="choose" value="'.$choose.'" />';
     echo "<hr width=\"640\" align=\"center\" noshade=\"noshade\" size=\"1\" />";
-    echo "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"640\" class=\"files\">";    
+    echo "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"640\" class=\"files\">";
     echo "<tr>";
     echo "<th width=\"5\"></th>";
     echo "<th align=\"left\" class=\"header name\">$strname</th>";
@@ -671,10 +711,7 @@ function displaydir ($wdir) {
     echo "<th align=\"right\" class=\"header commands\">$straction</th>";
     echo "</tr>\n";
 
-    if ($wdir == "/") {
-        $wdir = "";
-    }
-    if (!empty($wdir)) {
+    if ($wdir != "/") {
         $dirlist[] = '..';
     }
 
@@ -706,7 +743,7 @@ function displaydir ($wdir) {
                 print_cell("right", $filedate, 'date');
                 print_cell("right", "<a href=\"index.php?id=$id&amp;wdir=$wdir&amp;file=$filesafe&amp;action=rename&amp;choose=$choose\">$strrename</a>", 'commands');
             }
-    
+
             echo "</tr>";
         }
     }
@@ -720,31 +757,27 @@ function displaydir ($wdir) {
 
             $count++;
             $filename    = $fullpath."/".$file;
-            $fileurl     = "$wdir/$file";
+            $fileurl     = trim($wdir, "/")."/$file";
             $filesafe    = rawurlencode($file);
             $fileurlsafe = rawurlencode($fileurl);
             $filedate    = userdate(filemtime($filename), "%d %b %Y, %I:%M %p");
 
-            if (substr($fileurl,0,1) == '/') {
-                $selectfile = substr($fileurl,1);
-            } else {
-                $selectfile = $fileurl;
-            }
+            $selectfile = trim($fileurl, "/");
 
             echo "<tr class=\"file\">";
 
             print_cell("center", "<input type=\"checkbox\" name=\"file$count\" value=\"$fileurl\" />", 'checkbox');
             echo "<td align=\"left\" nowrap=\"nowrap\" class=\"name\">";
             if ($CFG->slasharguments) {
-                $ffurl = "/file.php/$id$fileurl";
+                $ffurl = str_replace('//', '/', "/file.php/$id/$fileurl");
             } else {
-                $ffurl = "/file.php?file=/$id$fileurl";
+                $ffurl = str_replace('//', '/', "/file.php?file=/$id/$fileurl");
             }
-            link_to_popup_window ($ffurl, "display", 
-                                  "<img src=\"$CFG->pixpath/f/$icon\" height=\"16\" width=\"16\" border=\"0\" alt=\"File\" />", 
+            link_to_popup_window ($ffurl, "display",
+                                  "<img src=\"$CFG->pixpath/f/$icon\" height=\"16\" width=\"16\" border=\"0\" alt=\"File\" />",
                                   480, 640);
             echo '&nbsp;';
-            link_to_popup_window ($ffurl, "display", 
+            link_to_popup_window ($ffurl, "display",
                                   htmlspecialchars($file),
                                   480, 640);
             echo "</td>";
@@ -771,21 +804,17 @@ function displaydir ($wdir) {
             }
 
             print_cell("right", "$edittext <a href=\"index.php?id=$id&amp;wdir=$wdir&amp;file=$filesafe&amp;action=rename&amp;choose=$choose\">$strrename</a>", 'commands');
-    
+
             echo "</tr>";
         }
     }
     echo "</table>";
     echo "<hr width=\"640\" align=\"center\" noshade=\"noshade\" size=\"1\" />";
 
-    if (empty($wdir)) {
-        $wdir = "/";
-    }
-
-    echo "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"640\">";    
+    echo "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"640\">";
     echo "<tr><td>";
     echo "<input type=\"hidden\" name=\"id\" value=\"$id\" />";
-    echo '<input type="hidden" name="choose" value="'.$choose.'">';
+    echo '<input type="hidden" name="choose" value="'.$choose.'" />';
     echo "<input type=\"hidden\" name=\"wdir\" value=\"$wdir\" /> ";
     echo "<input type=\"hidden\" name=\"sesskey\" value=\"$USER->sesskey\" />";
     $options = array (
@@ -801,7 +830,7 @@ function displaydir ($wdir) {
     echo "<td align=\"center\">";
     if (!empty($USER->fileop) and ($USER->fileop == "move") and ($USER->filesource <> $wdir)) {
         echo "<form action=\"index.php\" method=\"get\">";
-        echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+        echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
         echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
         echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
         echo " <input type=\"hidden\" name=\"action\" value=\"paste\" />";
@@ -809,18 +838,25 @@ function displaydir ($wdir) {
         echo " <input type=\"submit\" value=\"$strmovefilestohere\" />";
         echo "</form>";
     }
+    echo "</td>";
     echo "<td align=\"right\">";
         echo "<form action=\"index.php\" method=\"get\">";
-        echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+        echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
         echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
         echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
         echo " <input type=\"hidden\" name=\"action\" value=\"makedir\" />";
         echo " <input type=\"submit\" value=\"$strmakeafolder\" />";
         echo "</form>";
     echo "</td>";
+    echo "<td align=\"right\">";
+        echo "<form action=\"index.php\" method=\"get\">"; //dummy form - alignment only
+        echo " <input type=\"button\" value=\"$strselectall\" onclick=\"checkall();\" />";
+        echo " <input type=\"button\" value=\"$strselectnone\" onclick=\"uncheckall();\" />";
+        echo "</form>";
+    echo "</td>";
     echo "<td align=\"right\">";
         echo "<form action=\"index.php\" method=\"get\">";
-        echo ' <input type="hidden" name="choose" value="'.$choose.'">';
+        echo ' <input type="hidden" name="choose" value="'.$choose.'" />';
         echo " <input type=\"hidden\" name=\"id\" value=\"$id\" />";
         echo " <input type=\"hidden\" name=\"wdir\" value=\"$wdir\" />";
         echo " <input type=\"hidden\" name=\"action\" value=\"upload\" />";
@@ -832,4 +868,23 @@ function displaydir ($wdir) {
 
 }
 
+function files_get_cm_from_resource_name($clean_name) {
+    global $CFG;
+
+    $SQL =  'SELECT a.id  FROM '.$CFG->prefix.'course_modules a, '.$CFG->prefix.'resource b
+        WHERE a.instance = b.id AND b.reference = "'.$clean_name.'"';
+    $resource = get_record_sql($SQL);
+    return $resource->id;
+}
+
+function get_dir_name_from_resource($clean_name) {
+    global $CFG;
+
+    $LIKE    = sql_ilike();
+
+    $SQL  = 'SELECT * FROM '.$CFG->prefix.'resource WHERE reference '.$LIKE. "\"%$clean_name%\"";
+    $resource = get_records_sql($SQL);
+    return $resource;
+}
+
 ?>
