@@ -1,4 +1,4 @@
-<?PHP  //$Id: postgres7.php,v 1.140.2.7 2005/07/15 13:03:20 moodler Exp $
+<?PHP  //$Id: postgres7.php,v 1.184.2.11 2006/08/28 21:13:12 skodak Exp $
 //
 // This file keeps track of upgrades to Moodle.
 // 
@@ -543,7 +543,7 @@ function main_upgrade($oldversion=0) {
     if ($oldversion < 2004072901) {  // Fixing error in schema
         if ($record = get_record('log_display', 'module', 'course', 'action', 'update')) {
             delete_records('log_display', 'module', 'course', 'action', 'update');
-            insert_record('log_display', $record, false, 'module');
+            insert_record('log_display', $record, false);
         }
     }
 
@@ -849,8 +849,8 @@ function main_upgrade($oldversion=0) {
                             CREATE INDEX prefix_message_contacts_useridcontactid_idx ON prefix_message_contacts (userid,contactid);
                             ");
 
-        modify_database('',"INSERT INTO prefix_log_display VALUES ('message', 'write', 'user', 'firstname||\' \'||lastname');
-                            INSERT INTO prefix_log_display VALUES ('message', 'read', 'user', 'firstname||\' \'||lastname');
+        modify_database('',"INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'write', 'user', 'firstname||\' \'||lastname');
+                            INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'read', 'user', 'firstname||\' \'||lastname');
                             ");
     }
 
@@ -861,17 +861,17 @@ function main_upgrade($oldversion=0) {
        
                                 
     if ($oldversion < 2005010100) {
-        modify_database('',"INSERT INTO prefix_log_display VALUES ('message', 'add contact', 'user', 'firstname||\' \'||lastname');
-                            INSERT INTO prefix_log_display VALUES ('message', 'remove contact', 'user', 'firstname||\' \'||lastname');
-                            INSERT INTO prefix_log_display VALUES ('message', 'block contact', 'user', 'firstname||\' \'||lastname');
-                            INSERT INTO prefix_log_display VALUES ('message', 'unblock contact', 'user', 'firstname||\' \'||lastname');
+        modify_database('',"INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'add contact', 'user', 'firstname||\' \'||lastname');
+                            INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'remove contact', 'user', 'firstname||\' \'||lastname');
+                            INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'block contact', 'user', 'firstname||\' \'||lastname');
+                            INSERT INTO prefix_log_display (module, action, mtable, field) VALUES ('message', 'unblock contact', 'user', 'firstname||\' \'||lastname');
                             ");
     }
 
     if ($oldversion < 2005011000) {     // Create a .htaccess file in dataroot, just in case
         if (!file_exists($CFG->dataroot.'/.htaccess')) {
             if ($handle = fopen($CFG->dataroot.'/.htaccess', 'w')) {   // For safety
-                @fwrite($handle, "deny from all\r\n");
+                @fwrite($handle, "deny from all\r\nAllowOverride None\r\n");
                 @fclose($handle); 
                 notify("Created a default .htaccess file in $CFG->dataroot");
             }
@@ -881,9 +881,9 @@ function main_upgrade($oldversion=0) {
     if ($oldversion < 2005012500) { // add new table for meta courses.
         /*
         modify_database("","CREATE TABLE prefix_meta_course (
-	         id SERIAL primary key,
-	         parent_course integer NOT NULL,
-	         child_course integer NOT NULL
+             id SERIAL primary key,
+             parent_course integer NOT NULL,
+             child_course integer NOT NULL
              );");
 
         modify_database("","CREATE INDEX prefix_meta_course_parent_idx ON prefix_meta_course (parent_course);");
@@ -897,9 +897,9 @@ function main_upgrade($oldversion=0) {
         execute_sql("ALTER TABLE {$CFG->prefix}course DROP COLUMN meta_course",false); // drop silently
         
         modify_database("","CREATE TABLE prefix_course_meta (
-	         id SERIAL primary key,
-	         parent_course integer NOT NULL,
-	         child_course integer NOT NULL
+             id SERIAL primary key,
+             parent_course integer NOT NULL,
+             child_course integer NOT NULL
              );");
 
         modify_database("","CREATE INDEX prefix_course_meta_parent_idx ON prefix_course_meta (parent_course);");
@@ -1014,8 +1014,8 @@ function main_upgrade($oldversion=0) {
                               id SERIAL PRIMARY KEY,
                               courseid integer NOT NULL default '0',
                               letter varchar(8) NOT NULL default 'NA',
-                              grade_high decimal(4,2) NOT NULL default '100.00',
-                              grade_low decimal(4,2) NOT NULL default '0.00'
+                              grade_high decimal(6,2) NOT NULL default '100.00',
+                              grade_low decimal(6,2) NOT NULL default '0.00'
                             );");
 
         modify_database('',"CREATE INDEX prefix_grade_letter_courseid_idx ON prefix_grade_letter (courseid);");
@@ -1090,19 +1090,22 @@ function main_upgrade($oldversion=0) {
         table_column('grade_category', 'weight', 'weight', 'numeric(5,2)', '', '', '0.00', '', '');
     }
 
-    if ($oldversion < 2005052301 ) { // Add config_plugins table
+    if ($oldversion < 2005053000 ) { // Add config_plugins table
         
+        // this table was created on the MOODLE_15_STABLE branch
+        // so it may already exist. Therefore we hide potential errors
+        // (Postgres doesn't support CREATE TABLE IF NOT EXISTS)
         execute_sql("CREATE TABLE {$CFG->prefix}config_plugins (
                         id     SERIAL PRIMARY KEY,
                         plugin varchar(100) NOT NULL default 'core',
                         name   varchar(100) NOT NULL default '',
                         value  text NOT NULL default '',
                         CONSTRAINT {$CFG->prefix}config_plugins_plugin_name_uk UNIQUE (plugin, name)
-                     );");
+                     );", false);
 
     }
 
-    if ($oldversion < 2005052302) {  // migrate some config items to config_plugins table
+    if ($oldversion < 2005060200) {  // migrate some config items to config_plugins table
 
         // NOTE: this block is in both postgres AND mysql upgrade
         // files. If you edit either, update the otherone. 
@@ -1168,6 +1171,437 @@ function main_upgrade($oldversion=0) {
         }
     }
 
+    if ($oldversion < 2005060223) { // Mass cleanup of bad postgres upgrade scripts
+        execute_sql("DROP TABLE {$CFG->prefix}attendance_roll", false); // There are no attendance module anymore
+        modify_database('','ALTER TABLE prefix_config ALTER value SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_course ALTER metacourse SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_course ALTER theme SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_event ALTER repeatid SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_groups ALTER password SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_groups ALTER theme SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_message ALTER format SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_message_read ALTER format SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_groups ALTER theme SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user ALTER aim DROP DEFAULT');
+        modify_database('','ALTER TABLE prefix_user ALTER idnumber DROP DEFAULT');
+        modify_database('','ALTER TABLE prefix_user ALTER msn DROP DEFAULT');
+        modify_database('','ALTER TABLE prefix_user ALTER policyagreed SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user ALTER skype DROP DEFAULT');
+        modify_database('','ALTER TABLE prefix_user ALTER theme SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user ALTER timezone SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user ALTER trackforums SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user ALTER yahoo DROP DEFAULT');
+        modify_database('','ALTER TABLE prefix_user_students ALTER enrol SET NOT NULL');
+        modify_database('','ALTER TABLE prefix_user_teachers ALTER enrol SET NOT NULL');
+    }
+
+    if ($oldversion < 2005071700) {  // Close down the Dialogue module, we are removing it from CVS.
+        if (!file_exists($CFG->dirroot.'/mod/dialogue/lib.php')) {
+            if (count_records('dialogue')) {   // We have some data, so should keep it
+
+                set_field('modules', 'visible', 0, 'name', 'dialogue');
+                notify('The Dialogue module has been discontinued.  If you really want to 
+                        continue using it, you should download it individually from 
+                        http://download.moodle.org/modules and install it, then 
+                        reactivate it from Admin >> Configuration >> Modules.  
+                        None of your existing data has been deleted, so all existing 
+                        Dialogue activities should re-appear.');
+
+            } else {  // No data, so do a complete delete
+
+                execute_sql('DROP TABLE '.$CFG->prefix.'dialogue', false);
+                delete_records('modules', 'name', 'dialogue');
+                notify("The Dialogue module has been discontinued and removed from your site.  
+                        You weren't using it anyway.  ;-)");
+            }
+        }
+    }
+
+    if ($oldversion < 2005072000) {  // Add a couple fields to mdl_event to work towards iCal import/export
+        table_column('event', '', 'uuid', 'char', '36', '', '', 'not null', 'visible');
+        table_column('event', '', 'sequence', 'integer', '10', 'unsigned', '1', 'not null', 'uuid');
+    }
+    
+    if ($oldversion < 2005072100) { // run the online assignment cleanup code
+        include($CFG->dirroot.'/'.$CFG->admin.'/oacleanup.php');
+        if (function_exists('online_assignment_cleanup')) {
+            online_assignment_cleanup();
+        }
+    }
+
+    if ($oldversion < 2005072200) { // fix the mistakenly-added currency stuff from enrol/authorize
+        execute_sql("DROP TABLE {$CFG->prefix}currencies", false); // drop silently
+        execute_sql("ALTER TABLE {$CFG->prefix}course DROP currency", false);
+        $defaultcurrency = empty($CFG->enrol_currency) ? 'USD' : $CFG->enrol_currency;
+        table_column('course', '', 'currency', 'char', '3', '', $defaultcurrency, 'not null', 'cost');
+    }
+
+    if ($oldversion < 2005081600) { //set up the course requests table
+        modify_database('',"CREATE TABLE prefix_course_request (
+           id SERIAL PRIMARY KEY,
+           fullname varchar(254) NOT NULL default '',
+           shortname varchar(15) NOT NULL default '',
+           summary text NOT NULL default '',
+           reason text NOT NULL default '',
+           requester INTEGER NOT NULL default 0
+         );");
+        
+        modify_database('','CREATE INDEX prefix_course_request_shortname_idx ON prefix_course_request (shortname);');
+
+        table_column('course','','requested');
+    }
+
+    if ($oldversion < 2005081601) {
+        modify_database('','CREATE TABLE prefix_course_allowed_modules (
+            id SERIAL PRIMARY KEY,
+            course INTEGER NOT NULL default 0,
+            module INTEGER NOT NULL default 0
+         );');
+         
+        modify_database('','CREATE INDEX prefix_course_allowed_modules_course_idx ON prefix_course_allowed_modules (course);');
+        modify_database('','CREATE INDEX prefix_course_allowed_modules_module_idx ON prefix_course_allowed_modules (module);');
+        table_column('course','','restrictmodules','int','1','','0','not null');
+    }
+    
+    if ($oldversion < 2005081700) {
+        table_column('course_categories','','depth','integer');
+        table_column('course_categories','','path','varchar','255');
+    }
+
+    if  ($oldversion < 2005090100) { // stats!
+        modify_database('','CREATE TABLE prefix_stats_daily (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           students INTEGER NOT NULL default 0,
+           teachers INTEGER NOT NULL default 0,
+           activestudents INTEGER NOT NULL default 0,
+           activeteachers INTEGER NOT NULL default 0,
+           studentreads INTEGER NOT NULL default 0,
+           studentwrites INTEGER NOT NULL default 0,
+           teacherreads INTEGER NOT NULL default 0,
+           teacherwrites INTEGER NOT NULL default 0,
+           logins INTEGER NOT NULL default 0,
+           uniquelogins INTEGER NOT NULL default 0
+        );');
+
+        modify_database('','CREATE INDEX prefix_stats_daily_courseid_idx ON prefix_stats_daily (courseid);');
+        modify_database('','CREATE INDEX prefix_stats_daily_timeend_idx ON prefix_stats_daily (timeend);');
+        
+        modify_database('','CREATE TABLE prefix_stats_weekly (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           students INTEGER NOT NULL default 0,
+           teachers INTEGER NOT NULL default 0,
+           activestudents INTEGER NOT NULL default 0,
+           activeteachers INTEGER NOT NULL default 0,
+           studentreads INTEGER NOT NULL default 0,
+           studentwrites INTEGER NOT NULL default 0,
+           teacherreads INTEGER NOT NULL default 0,
+           teacherwrites INTEGER NOT NULL default 0,
+           logins INTEGER NOT NULL default 0,
+           uniquelogins INTEGER NOT NULL default 0
+        );');
+
+        modify_database('','CREATE INDEX prefix_stats_weekly_courseid_idx ON prefix_stats_weekly (courseid);');
+        modify_database('','CREATE INDEX prefix_stats_weekly_timeend_idx ON prefix_stats_weekly (timeend);');
+
+        modify_database('','CREATE TABLE prefix_stats_monthly (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           students INTEGER NOT NULL default 0,
+           teachers INTEGER NOT NULL default 0,
+           activestudents INTEGER NOT NULL default 0,
+           activeteachers INTEGER NOT NULL default 0,
+           studentreads INTEGER NOT NULL default 0,
+           studentwrites INTEGER NOT NULL default 0,
+           teacherreads INTEGER NOT NULL default 0,
+           teacherwrites INTEGER NOT NULL default 0,
+           logins INTEGER NOT NULL default 0,
+           uniquelogins INTEGER NOT NULL default 0
+        );');
+
+        modify_database('','CREATE INDEX prefix_stats_monthly_courseid_idx ON prefix_stats_monthly (courseid);');
+        modify_database('','CREATE INDEX prefix_stats_monthly_timeend_idx ON prefix_stats_monthly (timeend);');
+        
+        modify_database("","CREATE TABLE prefix_stats_user_daily (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           userid INTEGER NOT NULL default 0,
+           roleid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           statsreads INTEGER NOT NULL default 0,
+           statswrites INTEGER NOT NULL default 0,
+           stattype varchar(30) NOT NULL default ''
+         );");
+         
+         modify_database("","CREATE INDEX prefix_stats_user_daily_courseid_idx ON prefix_stats_user_daily (courseid);");
+         modify_database("","CREATE INDEX prefix_stats_user_daily_userid_idx ON prefix_stats_user_daily (userid);");
+         modify_database("","CREATE INDEX prefix_stats_user_daily_roleid_idx ON prefix_stats_user_daily (roleid);");
+         modify_database("","CREATE INDEX prefix_stats_user_daily_timeend_idx ON prefix_stats_user_daily (timeend);");
+
+         modify_database("","CREATE TABLE prefix_stats_user_weekly (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           userid INTEGER NOT NULL default 0,
+           roleid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           statsreads INTEGER NOT NULL default 0,
+           statswrites INTEGER NOT NULL default 0,
+           stattype varchar(30) NOT NULL default ''
+         );");
+         
+         modify_database("","CREATE INDEX prefix_stats_user_weekly_courseid_idx ON prefix_stats_user_weekly (courseid);");
+         modify_database("","CREATE INDEX prefix_stats_user_weekly_userid_idx ON prefix_stats_user_weekly (userid);");
+         modify_database("","CREATE INDEX prefix_stats_user_weekly_roleid_idx ON prefix_stats_user_weekly (roleid);");
+         modify_database("","CREATE INDEX prefix_stats_user_weekly_timeend_idx ON prefix_stats_user_weekly (timeend);");
+
+         modify_database("","CREATE TABLE prefix_stats_user_monthly (
+           id SERIAL PRIMARY KEY,
+           courseid INTEGER NOT NULL default 0,
+           userid INTEGER NOT NULL default 0,
+           roleid INTEGER NOT NULL default 0,
+           timeend INTEGER NOT NULL default 0,
+           statsreads INTEGER NOT NULL default 0,
+           statswrites INTEGER NOT NULL default 0,
+           stattype varchar(30) NOT NULL default ''
+         );");
+         
+         modify_database("","CREATE INDEX prefix_stats_user_monthly_courseid_idx ON prefix_stats_user_monthly (courseid);");
+         modify_database("","CREATE INDEX prefix_stats_user_monthly_userid_idx ON prefix_stats_user_monthly (userid);");
+         modify_database("","CREATE INDEX prefix_stats_user_monthly_roleid_idx ON prefix_stats_user_monthly (roleid);");
+         modify_database("","CREATE INDEX prefix_stats_user_monthly_timeend_idx ON prefix_stats_user_monthly (timeend);");
+    }
+    
+    if ($oldversion < 2005100300) {
+        table_column('course','','expirynotify','integer','1');
+        table_column('course','','expirythreshold','integer');
+        table_column('course','','notifystudents','integer','1');
+        $new = new stdClass();
+        $new->name = 'lastexpirynotify';
+        $new->value = 0;
+        insert_record('config', $new);
+    }
+
+    if ($oldversion < 2005100400) {
+        table_column('course','','enrollable','integer','1','unsigned','1');
+        table_column('course','','enrolstartdate','integer');
+        table_column('course','','enrolenddate','integer');
+    }
+
+
+    if ($oldversion < 2005101200) { // add enrolment key to course_request.
+        table_column('course_request','','password','text');
+    }
+
+    if ($oldversion < 2006030800) { # add extra indexes to log (see bug #4112)
+        modify_database('',"CREATE INDEX prefix_log_userid_idx ON prefix_log (userid);");
+        modify_database('',"CREATE INDEX prefix_log_info_idx ON prefix_log (info);");
+    }
+
+    if ($oldversion < 2006030900) {
+        table_column('course','','enrol','varchar','20','','');
+
+        if ($CFG->enrol == 'internal' || $CFG->enrol == 'manual') {
+            set_config('enrol_plugins_enabled', 'manual');
+            set_config('enrol', 'manual');
+        } else {
+            set_config('enrol_plugins_enabled', 'manual,'.$CFG->enrol);
+        }
+
+        require_once("$CFG->dirroot/enrol/enrol.class.php");
+        $defaultenrol = enrolment_factory::factory($CFG->enrol);
+        if (!method_exists($defaultenrol, 'print_entry')) { // switch enrollable to off for all courses in this case
+            modify_database('', 'UPDATE prefix_course SET enrollable = 0');
+        }
+
+        execute_sql("UPDATE {$CFG->prefix}user_students SET enrol='manual' WHERE enrol='' OR enrol='internal'");
+        execute_sql("UPDATE {$CFG->prefix}user_teachers SET enrol='manual' WHERE enrol=''");
+
+    }
+    
+    if ($oldversion < 2006031000) {
+
+        modify_database("","CREATE TABLE prefix_post (
+          id SERIAL PRIMARY KEY,
+          userid INTEGER NOT NULL default 0,
+          courseid INTEGER NOT NULL default 0,
+          groupid INTEGER NOT NULL default 0,
+          moduleid INTEGER NOT NULL default 0,
+          coursemoduleid INTEGER NOT NULL default 0,
+          subject varchar(128) NOT NULL default '',
+          summary text,
+          content text,
+          uniquehash varchar(128) NOT NULL default '',
+          rating INTEGER NOT NULL default 0,
+          format INTEGER NOT NULL default 0,
+          publishstate varchar(10) CHECK (publishstate IN ('draft','site','public')) NOT NULL default 'draft',
+          lastmodified INTEGER NOT NULL default '0',
+          created INTEGER NOT NULL default '0'
+        );");
+
+         modify_database("","CREATE INDEX id_user_idx ON prefix_post  (id, courseid);");
+         modify_database("","CREATE INDEX post_lastmodified_idx ON prefix_post (lastmodified);");
+         modify_database("","CREATE INDEX post_subject_idx ON prefix_post (subject);");
+
+         modify_database("","CREATE TABLE prefix_tags (
+          id SERIAL PRIMARY KEY,
+          type varchar(255) NOT NULL default 'official',
+          userid INTEGER NOT NULL default 0,
+          text varchar(255) NOT NULL default ''
+        );");
+
+         modify_database("","CREATE TABLE prefix_blog_tag_instance (
+          id SERIAL PRIMARY KEY,
+          entryid integer NOT NULL default 0,
+          tagid integer NOT NULL default 0,
+          groupid integer NOT NULL default 0,
+          courseid integer NOT NULL default 0,
+          userid integer NOT NULL default 0
+        );");
+    }
+
+    if ($oldversion < 2006031400) {
+        require_once("$CFG->dirroot/enrol/enrol.class.php");
+        $defaultenrol = enrolment_factory::factory($CFG->enrol);
+        if (!method_exists($defaultenrol, 'print_entry')) {
+            set_config('enrol', 'manual');
+        }
+    }
+
+    if ($oldversion < 2006032000) {
+        table_column('post','','module','varchar','20','','','not null', 'id');
+        modify_database('',"CREATE INDEX post_module_idx ON prefix_post (module);");
+        modify_database('',"UPDATE prefix_post SET module = 'blog';");
+    }
+
+    if ($oldversion < 2006032001) {
+        table_column('blog_tag_instance','','timemodified','integer','10','unsigned','0','not null', 'userid'); 
+        modify_database('',"CREATE INDEX bti_entryid_idx ON prefix_blog_tag_instance (entryid);");
+        modify_database('',"CREATE INDEX bti_tagid_idx ON prefix_blog_tag_instance (tagid);");
+        modify_database('',"UPDATE prefix_blog_tag_instance SET timemodified = '".time()."';");
+    }
+
+    if ($oldversion < 2006040500) { // Add an index to course_sections that was never upgraded (bug 5100)
+        execute_sql(" CREATE INDEX {$CFG->prefix}course_sections_coursesection_idx ON {$CFG->prefix}course_sections (course,section) ", false);
+    }
+
+    if ($oldversion < 2006041100) {
+        table_column('course_modules','','visibleold','integer','1','unsigned','1','not null', 'visible');
+    }
+
+    if ($oldversion < 2006042400) {
+        // Look through table log_display and get rid of duplicates.
+        $rs = get_recordset_sql('SELECT DISTINCT * FROM '.$CFG->prefix.'log_display');
+        
+        // Drop the log_display table and create it back with an id field.
+        execute_sql("DROP TABLE {$CFG->prefix}log_display", false);
+        
+        modify_database('', "CREATE TABLE prefix_log_display (
+                               id SERIAL PRIMARY KEY,
+                               module varchar(30) NOT NULL default '',
+                               action varchar(40) NOT NULL default '',
+                               mtable varchar(30) NOT NULL default '',
+                               field varchar(50) NOT NULL default '')");
+        
+        // Add index to ensure that module and action combination is unique.
+        modify_database('', 'CREATE INDEX prefix_log_display_moduleaction ON prefix_log_display (module,action)');
+        
+        // Insert the records back in, sans duplicates.
+        if ($rs && $rs->RecordCount() > 0) {
+            while (!$rs->EOF) {
+                $sql = "INSERT INTO {$CFG->prefix}log_display ".
+                            "VALUES('', '".$rs->fields['module']."', ".
+                            "'".$rs->fields['action']."', ".
+                            "'".$rs->fields['mtable']."', ".
+                            "'".$rs->fields['field']."')";
+                
+                execute_sql($sql, false);
+                $rs->MoveNext();
+            }
+        }
+    }
+    
+    // add 2 indexes to tags table
+    if ($oldversion < 2006042401) {
+        modify_database('',"CREATE INDEX tags_typeuserid_idx ON prefix_tags (type, userid);");
+        modify_database('',"CREATE INDEX tags_text_idx ON prefix_tags (text);");
+    }
+    
+    if ($oldversion < 2006050500) {
+        table_column('log', 'action', 'action', 'varchar', '40', '', '', 'not null');
+    }
+
+    if ($oldversion < 2006050502) {  // Close down the Dialogue module, we are removing it from CVS.
+        if (!file_exists($CFG->dirroot.'/mod/dialogue/lib.php')) {
+            if (!count_records('dialogue_conversations')) {   // no data, drop the extra tables
+                execute_sql('DROP TABLE '.$CFG->prefix.'dialogue_conversations', false);
+                execute_sql('DROP TABLE '.$CFG->prefix.'dialogue_entries', false);
+                notify("The Dialogue module has been discontinued and removed from your site.  
+                        You weren't using it anyway.  ;-)");
+            }
+        }
+
+        table_column('course_request', 'password', 'password', 'varchar', '50', '', '');
+
+        table_column('course', 'currency', 'currency', 'varchar', '3');
+
+        modify_database('', 'ALTER TABLE prefix_course_categories
+            ALTER COLUMN path SET DEFAULT \'\'');
+
+        table_column('log_display', 'module', 'module', 'varchar', '20');
+
+        modify_database("","DROP INDEX id_user_idx");
+        modify_database("","DROP INDEX post_lastmodified_idx");
+        modify_database("","DROP INDEX post_subject_idx");
+        modify_database('',"DROP INDEX bti_entryid_idx");
+        modify_database('',"DROP INDEX bti_tagid_idx");
+        modify_database('',"DROP INDEX post_module_idx");
+        modify_database('',"DROP INDEX tags_typeuserid_idx");
+        modify_database('',"DROP INDEX tags_text_idx");
+
+        modify_database("","CREATE INDEX {$CFG->prefix}id_user_idx           ON prefix_post (id, courseid);");
+        modify_database("","CREATE INDEX {$CFG->prefix}post_lastmodified_idx ON prefix_post (lastmodified);");
+        modify_database("","CREATE INDEX {$CFG->prefix}post_subject_idx      ON prefix_post (subject);");
+        modify_database('',"CREATE INDEX {$CFG->prefix}bti_entryid_idx       ON prefix_blog_tag_instance (entryid);");
+        modify_database('',"CREATE INDEX {$CFG->prefix}bti_tagid_idx         ON prefix_blog_tag_instance (tagid);");
+        modify_database('',"CREATE INDEX {$CFG->prefix}post_module_idx       ON prefix_post (moduleid);");
+        modify_database('',"CREATE INDEX {$CFG->prefix}tags_typeuserid_idx   ON prefix_tags (type, userid);");
+        modify_database('',"CREATE INDEX {$CFG->prefix}tags_text_idx         ON prefix_tags (text);");
+
+    }
+
+    // renaming of reads and writes for stats_user_xyz
+    if ($oldversion < 2006050503) { // change this later
+
+        // we are using this because we want silent updates
+
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_daily RENAME COLUMN reads TO statsreads", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_daily RENAME COLUMN writes TO statswrites", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_weekly RENAME COLUMN reads TO statsreads", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_weekly RENAME COLUMN writes TO statswrites", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_monthly RENAME COLUMN reads TO statsreads", false);
+        execute_sql("ALTER TABLE {$CFG->prefix}stats_user_monthly RENAME COLUMN writes TO statswrites", false);
+
+    }
+
+    // Adding some missing log actions
+    if ($oldversion < 2006050504) {
+        execute_sql("INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('course', 'report log', 'course', 'fullname')");
+        execute_sql("INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('course', 'report live', 'course', 'fullname')");
+        execute_sql("INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('course', 'report outline', 'course', 'fullname')");
+        execute_sql("INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('course', 'report participation', 'course', 'fullname')");
+        execute_sql("INSERT INTO {$CFG->prefix}log_display (module, action, mtable, field) VALUES ('course', 'report stats', 'course', 'fullname')");
+    }
+
+    //Renaming lastIP to lastip (all fields lowercase)
+    if ($oldversion < 2006050505) {
+        //Not needed unded PG because it stores fieldnames lowecase by default
+        //table_column("user", "lastIP", "lastip", "varchar", "15", "", "", "", "currentlogin");
+    }
+    
     return $result;
 }
 
