@@ -1,8 +1,12 @@
-<?php // $Id: index.php,v 1.74.2.6 2005/12/12 03:16:42 mjollnir_ Exp $
+<?php // $Id: index.php,v 1.88.2.5 2006/07/06 10:09:34 skodak Exp $
 
     require_once("../config.php");
 
-    $loginguest = optional_param('loginguest', false); // determines whether visitors are logged in as guest automatically
+    $loginguest  = optional_param('loginguest', 0, PARAM_BOOL); // determines whether visitors are logged in as guest automatically
+    $testcookies = optional_param('testcookies', 0, PARAM_BOOL); // request cookie test
+
+    //initialize variables
+    $errormsg = '';
 
 /// Check for timed out sessions
     if (!empty($SESSION->has_timed_out)) {
@@ -19,7 +23,7 @@
     if (! record_exists("user", "username", "guest")) {
         $guest->auth        = "manual"; 
         $guest->username    = "guest"; 
-        $guest->password    = md5("guest");
+        $guest->password    = hash_internal_user_password("guest");
         $guest->firstname   = addslashes(get_string("guestuser"));
         $guest->lastname    = " ";
         $guest->email       = "root@localhost";
@@ -39,13 +43,14 @@
         require($CFG->dirroot.'/auth/cas/login.php');
     }
 
-    if ($CFG->auth == 'shibboleth') {
-        if (!empty($SESSION->shibboleth_checked) ) {  // Just come from there
-            unset($SESSION->shibboleth_checked);
-        } else if (empty($_POST)) {                   // No incoming data, so redirect
-            redirect($CFG->wwwroot.'/auth/shibboleth/index.php');
-        }
-    }
+//  See http://moodle.org/mod/forum/discuss.php?d=39918#187611
+//    if ($CFG->auth == 'shibboleth') {
+//        if (!empty($SESSION->shibboleth_checked) ) {  // Just come from there
+//            unset($SESSION->shibboleth_checked);
+//        } else if (empty($_POST)) {                   // No incoming data, so redirect
+//            redirect($CFG->wwwroot.'/auth/shibboleth/index.php');
+//        }
+//    }
     
 
     
@@ -78,10 +83,10 @@
         // Handles the case of another Moodle site linking into a page on this site
         include($CFG->dirroot.'/login/weblinkauth.php');
         if (function_exists(weblink_auth)) {
-        	$user = weblink_auth($SESSION->wantsurl);
+            $user = weblink_auth($SESSION->wantsurl);
         }
         if ($user) {
-        	$frm->username = $user->username;
+            $frm->username = $user->username;
         } else {
             $frm = data_submitted($loginurl);
         }
@@ -91,11 +96,11 @@
 
 /// Check if the user has actually submitted login data to us
 
-    if ($frm and (get_moodle_cookie() == '') and ($frm->username!='guest') and !$user and empty($CFG->alternateloginurl)) {    // Login without cookie
+    if (empty($CFG->usesid) and $testcookies and (get_moodle_cookie() == '')) {    // Login without cookie when test requested
 
         $errormsg = get_string("cookiesnotenabled");
 
-    } else if ($frm) {                             // Login WITH cookies
+    } else  if ($frm) {                             // Login WITH cookies
 
         $frm->username = trim(moodle_strtolower($frm->username));
 
@@ -105,8 +110,8 @@
                 $errormsg = get_string('username').': '.get_string("alphanumerical");
                 $user = null;
             }
-        } 
-        
+        }
+
         if (($frm->username == 'guest') and empty($CFG->guestloginbutton)) {
             $user = false;    /// Can't log in as guest if guest button is disabled
             $frm = false;
@@ -143,11 +148,13 @@
                 $passwordchangeurl=$CFG->wwwroot.'/login/change_password.php';
             } elseif($CFG->changepassword) {
                 $passwordchangeurl=$CFG->changepassword;
-            } 
+            } else {
+                $passwordchangeurl = '';
+            }
             
             // check whether the user should be changing password
             if (get_user_preferences('auth_forcepasswordchange', false) || $frm->password == 'changeme'){
-                if (isset($passwordchangeurl)) {
+                if ($passwordchangeurl != '') {
                     redirect($passwordchangeurl);
                 } else {
                     error("You cannot proceed without changing your password. 
@@ -157,6 +164,7 @@
             }
 
 
+          /// Prepare redirection
             if (user_not_fully_set_up($USER)) {
                 $urltogo = $CFG->wwwroot.'/user/edit.php?id='.$USER->id.'&amp;course='.SITEID;
                 // We don't delete $SESSION->wantsurl yet, so we get there later
@@ -166,10 +174,19 @@
                 unset($SESSION->wantsurl);
 
             } else {
-                $urltogo = $CFG->wwwroot.'/';      /// Go to the standard home page
-                unset($SESSION->wantsurl);         /// Just in case
+                // no wantsurl stored or external - go to homepage
+                $urltogo = $CFG->wwwroot;
+                unset($SESSION->wantsurl);
             }
 
+          /// Go to my-moodle page instead of homepage if mymoodleredirect enabled
+            if (!isadmin() and !empty($CFG->mymoodleredirect) and !isguest()) {
+                if ($urltogo == $CFG->wwwroot or $urltogo == $CFG->wwwroot.'/' or $urltogo == $CFG->wwwroot.'/index.php') {
+                    $urltogo = $CFG->wwwroot.'/my/';
+                }
+            }
+
+
             // check if user password has expired
             // Currently supported only for ldap-authentication module
             if (isset($CFG->ldap_expiration) && $CFG->ldap_expiration == 1 ) {
@@ -208,7 +225,12 @@
 /// First, let's remember where the user was trying to get to before they got here
 
     if (empty($SESSION->wantsurl)) {
-        $SESSION->wantsurl = array_key_exists('HTTP_REFERER',$_SERVER) ? $_SERVER["HTTP_REFERER"] : $CFG->wwwroot; 
+        $SESSION->wantsurl = (array_key_exists('HTTP_REFERER',$_SERVER) && 
+                              $_SERVER["HTTP_REFERER"] != $CFG->wwwroot && 
+                              $_SERVER["HTTP_REFERER"] != $CFG->wwwroot.'/' &&
+                              $_SERVER["HTTP_REFERER"] != $CFG->httpswwwroot.'/login/' &&
+                              $_SERVER["HTTP_REFERER"] != $CFG->httpswwwroot.'/login/index.php')
+            ? $_SERVER["HTTP_REFERER"] : NULL;
     }
 
     if (!empty($loginurl)) {   // We don't want the standard forms, go elsewhere
@@ -218,10 +240,6 @@
 
 /// Generate the login page with forms
 
-    if (empty($errormsg)) {
-        $errormsg = '';
-    }
-    
     if ($session_has_timed_out) {
         $errormsg = get_string('sessionerroruser', 'error');
     }
@@ -230,7 +248,7 @@
         set_moodle_cookie('nobody');   // To help search for cookies
     }
     
-    if (empty($frm->username)) {
+    if (empty($frm->username) && $CFG->auth != 'shibboleth') {  // See bug 5184
         $frm->username = get_moodle_cookie() === 'nobody' ? '' : get_moodle_cookie();
         $frm->password = "";
     }
@@ -240,8 +258,11 @@
     } else {
         $focus = "login.username";
     }
-
-    if ($CFG->auth == "email" or $CFG->auth == "none" or chop($CFG->auth_instructions) <> "" ) {
+    
+    if (isset($CFG->auth_instructions)) {
+        $CFG->auth_instructions = trim($CFG->auth_instructions);
+    }
+    if ($CFG->auth == "email" or $CFG->auth == "none" or !empty($CFG->auth_instructions)) {
         $show_instructions = true;
     } else {
         $show_instructions = false;
