@@ -1,4 +1,4 @@
-<?php // $Id: discuss.php,v 1.54.2.5 2005/12/22 10:04:42 moodler Exp $
+<?php // $Id: discuss.php,v 1.65.2.7 2006/09/04 22:00:26 gustav_delius Exp $
 
 //  Displays a post, and all the posts below it.
 //  If no post is given, displays all posts in a discussion
@@ -32,6 +32,15 @@
             error("You must be a $course->teacher to view this forum");
         }
 
+    } elseif ($forum->type == "news") {
+        if (!((isadmin() and !empty($CFG->admineditalways))
+            || isteacher($course->id)
+            || (!empty($USER->id) && $USER->id == $discussion->userid)
+            || (($discussion->timestart == 0 || $discussion->timestart <= time())
+            && ($discussion->timeend == 0 || $discussion->timeend > time())))) {
+            error('Discussion ID was incorrect or no longer exists', "$CFG->wwwroot/mod/forum/view.php?f=$forum->id");
+        }
+
     } else {
         if (! $cm = get_coursemodule_from_instance("forum", $discussion->forum, $course->id)) {
             error("Course Module ID was incorrect");
@@ -48,8 +57,29 @@
             if (!forum_move_attachments($discussion, $move)) {
                 notify("Errors occurred while moving attachment directories - check your file permissions");
             }
-            set_field("forum_discussions", "forum", $forum->id, "id", $discussion->id);
+            
+            if (!$fromforum = get_record("forum", "id", $discussion->forum)) {
+                notify('Bad forum ID stored in this discussion');
+            }
             $discussion->forum = $forum->id;
+            // Had to comment out the following line because the timemodified field is used to sort the list
+            // of discussions and also to provide the "Last post" date. It should really be renamed to timelastpost.
+            // $discussion->timemodified = time();
+            
+            if (update_record('forum_discussions', $discussion)) {
+                // Update RSS feeds for both from and to forums.
+                require_once('rsslib.php');
+                require_once($CFG->libdir.'/rsslib.php');
+                
+                // Delete the RSS files for the 2 forums because we want to force
+                // the regeneration of the feeds since the discussions have been
+                // moved.
+                if (!forum_rss_delete_file($forum) || !forum_rss_delete_file($fromforum)) {
+                    notify('Could not purge the cached RSS feeds for the source and/or'.
+                           'destination forum(s) - check your file permissionsforums');
+                }
+            }
+            
             if ($cm = get_coursemodule_from_instance("forum", $forum->id, $course->id)) {
                 add_to_log($course->id, "forum", "move discussion", "discuss.php?d=$discussion->id", "$discussion->id",
                            $cm->id);
@@ -62,7 +92,6 @@
         }
     }
 
-
     $logparameters = "d=$discussion->id";
     if ($parent) {
         $logparameters .= "&amp;parent=$parent";
@@ -95,9 +124,8 @@
         $navtail = format_string($discussion->name);
     }
     
-    //check if user can view this post
-    if (!forum_user_can_view_post($parent,$course)){
-        error('You do not have permissions to view this post');
+    if (!forum_user_can_view_post($parent, $course, $cm, $forum, $discussion)) {
+        error('You do not have permissions to view this post', "$CFG->wwwroot/mod/forum/view.php?f=$forum->id");
     }
 
     if (! $post = forum_get_post_full($parent)) {
@@ -146,12 +174,12 @@
                                        
     
     if ($groupmode and !isteacheredit($course->id)) {   // Groups must be kept separate
-        $mygroupid = mygroupid($course->id);
-
+        //change this to ismember
+        $mygroupid = mygroupid($course->id);//only useful if 0, otherwise it's an array now
         if ($groupmode == SEPARATEGROUPS) {
             require_login();
 
-            if ((empty($mygroupid) and $discussion->groupid == -1) || ($mygroupid == $discussion->groupid)) {
+            if ((empty($mygroupid) and $discussion->groupid == -1) || (ismember($discussion->groupid) || $mygroupid == $discussion->groupid)) {
                 $canreply = true;
             } elseif ($discussion->groupid == -1) {
                 $canreply = false;
@@ -162,7 +190,7 @@
             }
 
         } else if ($groupmode == VISIBLEGROUPS) {
-            $canreply = ((empty($mygroupid) and $discussion->groupid == -1) || ($mygroupid == $discussion->groupid));
+            $canreply = ((empty($mygroupid) and $discussion->groupid == -1) || (ismember($discussion->groupid) || $mygroupid == $discussion->groupid));
         }
     }
 
@@ -210,11 +238,20 @@
     }
     echo "</td></tr></table>";
 
+    if (!empty($forum->blockafter) && !empty($forum->blockperiod)) {
+        $a->blockafter = $forum->blockafter;
+        $a->blockperiod = get_string('secondstotime'.$forum->blockperiod);
+        notify(get_string('thisforumisthrottled','forum',$a));
+    }
+
+    if ($forum->type == 'qanda' && !isteacher($forum->course) && !forum_user_has_posted($forum->id,$discussion->id,$USER->id)) {
+        notify(get_string('qandanotify','forum'));
+    }
+
     if (isset($discussionmoved)) {
         notify(get_string("discussionmoved", "forum", format_string($forum->name,true)));
     }
 
-
 /// Print the actual discussion
 
     forum_print_discussion($course, $forum, $discussion, $post, $displaymode, $canreply);
