@@ -1,4 +1,4 @@
-<?php // $Id: post.php,v 1.81.2.2 2005/11/25 07:09:04 moodler Exp $
+<?php // $Id: post.php,v 1.94.2.6 2006/07/31 10:04:18 skodak Exp $
 
 //  Edit and save a new post to a discussion
 
@@ -6,14 +6,20 @@
     require_once('lib.php');
 
     $reply = optional_param('reply', 0, PARAM_INT);
+    $forum = optional_param('forum', 0, PARAM_INT);
+    $edit = optional_param('edit', 0, PARAM_INT);
+    $delete = optional_param('delete', 0, PARAM_INT);
+    $prune = optional_param('prune',0,PARAM_INT);
+    $name = optional_param('name','',PARAM_CLEAN);
+    $confirm = optional_param('confirm',0,PARAM_INT);
 
     if (isguest()) {
         $wwwroot = $CFG->wwwroot.'/login/index.php';
         if (!empty($CFG->loginhttps)) {
-            $wwwroot = str_replace('http','https', $wwwroot);
+            $wwwroot = str_replace('http:','https:', $wwwroot);
         }
 
-        if (isset($forum)) {      // User is starting a new discussion in a forum
+        if (!empty($forum)) {      // User is starting a new discussion in a forum
             if (! $forum = get_record('forum', 'id', $forum)) {
                 error('The forum number was incorrect');
             }
@@ -55,78 +61,168 @@
     require_login(0, false);   // Script is useless unless they're logged in
 
     if ($post = data_submitted()) {
-        if (empty($post->course)) {
-            error('No course was defined!');
+        if (! $forum = get_record('forum', 'id', $forum)) {
+            error('The forum number was incorrect');
         }
 
-        if (!$course = get_record('course', 'id', $post->course)) {
+        if (!$course = get_record('course', 'id', $forum->course)) {
             error('Could not find specified course!');
         }
 
+        require_login($course->id, false);
+        $adminedit = (isadmin() and !empty($CFG->admineditalways));
+
         if (!empty($course->lang)) {           // Override current language
             $CFG->courselang = $course->lang;
         }
 
         if (empty($SESSION->fromurl)) {
-            $errordestination = "$CFG->wwwroot/mod/forum/view.php?f=$post->forum";
+            $errordestination = "$CFG->wwwroot/mod/forum/view.php?f=$forum->id";
         } else {
             $errordestination = $SESSION->fromurl;
         }
 
-        $post->subject = strip_tags($post->subject, '<lang><span>');        // Strip all tags except lang
+        $post->subject = clean_param(strip_tags($post->subject, '<lang><span>'), PARAM_CLEAN); // Strip all tags except multilang
 
-        //$post->message = clean_text($post->message, $post->format);   // Clean up any bad tags
+        //$post->message will be cleaned later before display
 
         $post->attachment = isset($_FILES['attachment']) ? $_FILES['attachment'] : NULL;
 
-        if (!$cm = get_coursemodule_from_instance("forum", $post->forum, $course->id)) { // For the logs
+        if (!$cm = get_coursemodule_from_instance("forum", $forum->id, $course->id)) { // For the logs
             $cm->id = 0;
         }
 
-        if (!$post->subject or !$post->message) {
+        if (($post->subject == '') or ($post->message == '')) {
             $post->error = get_string("emptymessage", "forum");
 
-        } else if ($post->edit) {           // Updating a post
-            $post->id = $post->edit;
+        } else if ($post->edit) {
+/// Updating a post
+           if (! $oldpost = forum_get_post_full($post->edit)) {
+	            error("Post ID was incorrect");
+	        }
+	        if (($oldpost->userid <> $USER->id) and !$adminedit) {
+	            error("You can't edit other people's posts!");
+	        }
+	        if (! $discussion = get_record("forum_discussions", "id", $oldpost->discussion)) {
+	            error("This post is not part of a discussion!");
+	        }
+	        if ($discussion->forum != $forum->id) {
+	            error("The forum number is incorrect");
+	        }
+	        if ($discussion->course != $course->id) {
+	            error("The course number is incorrect");
+	        }
+	        if (!($forum->type == 'news' && !$oldpost->parent && $discussion->timestart > time())) {
+	            if (((time() - $oldpost->created) > $CFG->maxeditingtime) and !$adminedit) {
+	                error( get_string("maxtimehaspassed", "forum", format_time($CFG->maxeditingtime)) );
+	            }
+	        }
+
+            $updatepost = new object;
+            $updatepost->id = $oldpost->id;
+            $updatepost->parent = $oldpost->parent;
+            $updatepost->forum = $oldpost->forum;
+            $updatepost->discussion = $oldpost->discussion;
+            $updatepost->userid = $oldpost->userid;
+
+            $updatepost->subject = $post->subject; //already cleaned
+            $updatepost->message = $post->message; //cleaning only before display
+            $updatepost->format = $post->format;
+            $updatepost->attachment = $post->attachment;
+
+            $updatepost->course = $course->id;
+            $updatepost->subscribe = optional_param('subscribe', 0, PARAM_BOOL);
+            $updatepost->unsubscribe = optional_param('unsubscribe', 0, PARAM_BOOL);
+
             $message = '';
-            
-            //fix for bug #4314
-            if (!$realpost = get_record('forum_posts','id',$post->id)){
-                $realpost = new object;
-                $realpost->userid = -1;
-            }
-            
-            if ($realpost->userid <> $USER->id && !isadmin()){
-                error ("you can not update this post");
-            }
-            
-            if (forum_update_post($post,$message)) {
 
-                add_to_log($course->id, "forum", "update post",
-                          "discuss.php?d=$post->discussion&amp;parent=$post->id", "$post->id", $cm->id);
-
-                $timemessage = 2;
-                if (!empty($message)) { // if we're printing stuff about the file upload
-                    $timemessage = 4;
+            if (get_field('forum', 'type', 'id', $forum) == 'news' && !$oldpost->parent) {
+                $updatediscussion = new object;
+                $updatediscussion->id = $oldpost->discussion;
+                if (empty($post->timestartdisabled)) {
+                    $updatediscussion->timestart = make_timestamp($post->timestartyear, $post->timestartmonth, $post->timestartday);
+                } else {
+                    $updatediscussion->timestart = 0;
                 }
-                $message .= '<br />'.get_string("postupdated", "forum");
-
-                if ($subscribemessage = forum_post_subscription($post)) {
-                    $timemessage = 4;
+                if (empty($post->timeenddisabled)) {
+                    $updatediscussion->timeend = make_timestamp($post->timeendyear, $post->timeendmonth, $post->timeendday);
+                } else {
+                    $updatediscussion->timeend = 0;
+                }
+                if (empty($post->timeenddisabled) && $updatediscussion->timeend <= $updatediscussion->timestart) {
+                    $post->error = get_string('timestartenderror', 'forum');
+                } elseif (!update_record('forum_discussions', $updatediscussion)) {
+                    error(get_string("couldnotupdate", "forum"), $errordestination);
                 }
-                redirect(forum_go_back_to("discuss.php?d=$post->discussion#$post->id"), $message.$subscribemessage, $timemessage);
-
-            } else {
-                error(get_string("couldnotupdate", "forum"), $errordestination);
             }
-            exit;
 
-        } else if ($post->discussion) { // Adding a new post to an existing discussion
+            if (!isset($post->error)) {
+
+                if (forum_update_post($updatepost,$message)) {
+
+                    add_to_log($course->id, "forum", "update post",
+                            "discuss.php?d=$updatepost->discussion&amp;parent=$updatepost->id", "$updatepost->id", $cm->id);
+
+                    $timemessage = 2;
+                    if (!empty($message)) { // if we're printing stuff about the file upload
+                        $timemessage = 4;
+                    }
+                    $message .= '<br />'.get_string("postupdated", "forum");
+
+                    if ($subscribemessage = forum_post_subscription($updatepost)) {
+                        $timemessage = 4;
+                    }
+                    redirect(forum_go_back_to("discuss.php?d=$updatepost->discussion#$updatepost->id"), $message.$subscribemessage, $timemessage);
+
+                } else {
+                    error(get_string("couldnotupdate", "forum"), $errordestination);
+                }
+                exit;
+
+            }
+         } else if ($post->discussion) {
+/// Adding a new post to an existing discussion
+	        if (! $discussion = get_record("forum_discussions", "id", $post->discussion)) {
+	            error("This post is not part of a discussion!");
+	        }
+	        if ($discussion->forum != $forum->id) {
+	            error("The forum number is incorrect");
+	        }
+	        if ($discussion->course != $course->id) {
+	            error("The course number is incorrect");
+	        }
+	        if (! $parent = forum_get_post_full($post->parent)) {
+	            error("Parent post does not exist");
+	        }
+            if ($parent->discussion != $discussion->id) {
+	            error("Parent not in this discussion");
+            }
+            if (! forum_user_can_post($forum)) {
+                error("Sorry, but you can not post in this forum.");
+            }
+
+
+            $newpost = new object;
+            $newpost->parent = $post->parent;
+            $newpost->forum = $forum->id;
+            $newpost->discussion = $discussion->id;
+            $newpost->parent = $parent->id;
+
+            $newpost->subject = $post->subject; //already cleaned
+            $newpost->message = $post->message; //cleaning only before display
+            $newpost->format = $post->format;
+            $newpost->mailnow = optional_param('mailnow', 0, PARAM_BOOL);
+
+            $newpost->course = $course->id;
+            $newpost->subscribe = optional_param('subscribe', 0, PARAM_BOOL);
+            $newpost->unsubscribe = optional_param('unsubscribe', 0, PARAM_BOOL);
+
             $message = '';
-            if ($post->id = forum_add_new_post($post,$message)) {
+
+            if ($newpost->id = forum_add_new_post($newpost,$message)) {
 
                 add_to_log($course->id, "forum", "add post",
-                          "discuss.php?d=$post->discussion&amp;parent=$post->id", "$post->id", $cm->id);
+                          "discuss.php?d=$newpost->discussion&amp;parent=$newpost->id", "$newpost->id", $cm->id);
 
                 $timemessage = 2;
                 if (!empty($message)) { // if we're printing stuff about the file upload
@@ -134,53 +230,106 @@
                 }
                 $message .= '<br />'.get_string("postadded", "forum", format_time($CFG->maxeditingtime));
 
-                if ($subscribemessage = forum_post_subscription($post)) {
+                if ($subscribemessage = forum_post_subscription($newpost)) {
                     $timemessage = 4;
                 }
 
-                redirect(forum_go_back_to("discuss.php?d=$post->discussion#$post->id"), $message.$subscribemessage, $timemessage);
+                if ($newpost->mailnow) {
+                    $message .= get_string("postmailnow", "forum");
+                    $timemessage = 4;
+                }
+
+                redirect(forum_go_back_to("discuss.php?d=$newpost->discussion#$newpost->id"), $message.$subscribemessage, $timemessage);
 
             } else {
                 error(get_string("couldnotadd", "forum"), $errordestination);
             }
             exit;
 
-        } else {                     // Adding a new discussion
-            $discussion = $post;
-            $discussion->name  = $post->subject;
+        } else {
+/// Adding a new discussion
+            if (! forum_user_can_post_discussion($forum)) {
+                error("Sorry, but you can not post a new discussion in this forum.");
+            }
+
+            $discussion = new object;
+            $discussion->forum = $forum->id;
+            $discussion->course = $course->id;
+
+            $discussion->mailnow = optional_param('mailnow', 0, PARAM_BOOL);
+            $discussion->name = $post->subject;
             $discussion->intro = $post->message;
-            $message = '';
-            if ($discussion->id = forum_add_discussion($discussion,$message)) {
+            $discussion->format = $post->format;
+            $discussion->groupid = get_current_group($course->id);
+            if (isteacheredit($course->id) and $discussion->groupid == 0) {
+                $discussion->groupid = -1;
+            }
 
-                add_to_log($course->id, "forum", "add discussion",
-                           "discuss.php?d=$discussion->id", "$discussion->id", $cm->id);
+            $discussion->course = $course->id;
+            $discussion->subscribe = optional_param('subscribe', 0, PARAM_BOOL);
+            $discussion->unsubscribe = optional_param('unsubscribe', 0, PARAM_BOOL);
 
-                $timemessage = 2;
-                if (!empty($message)) { // if we're printing stuff about the file upload
-                    $timemessage = 4;
-                }
-                $message .= '<br />'.get_string("postadded", "forum", format_time($CFG->maxeditingtime));
+            if (! forum_user_can_post_discussion($forum)) {
+                error("Sorry, but you can not post a new discussion in this forum.");
+            }
+
+            $newstopic = false;
+            if (get_field('forum', 'type', 'id', $forum) == 'news') {
+                $newstopic = true;
+            }
+            if ($newstopic && empty($post->timestartdisabled)) {
+                $discussion->timestart = make_timestamp($post->timestartyear, $post->timestartmonth, $post->timestartday);
+            } else {
+                $discussion->timestart = 0;
+            }
+            if ($newstopic && empty($post->timeenddisabled)) {
+                $discussion->timeend = make_timestamp($post->timeendyear, $post->timeendmonth, $post->timeendday);
+            } else {
+                $discussion->timeend = 0;
+            }
+            if ($newstopic && empty($post->timeenddisabled) && $discussion->timeend <= $discussion->timestart) {
+                $post->error = get_string('timestartenderror', 'forum');
+            } else {
+                $message = '';
+                if ($discussion->id = forum_add_discussion($discussion,$message)) {
+
+                    add_to_log($course->id, "forum", "add discussion",
+                            "discuss.php?d=$discussion->id", "$discussion->id", $cm->id);
+
+                    $timemessage = 2;
+                    if (!empty($message)) { // if we're printing stuff about the file upload
+                        $timemessage = 4;
+                    }
+                    $message .= '<br />'.get_string("postadded", "forum", format_time($CFG->maxeditingtime));
+
+                    if ($discussion->mailnow) {
+                        $message .= get_string("postmailnow", "forum");
+                        $timemessage = 4;
+                    }
+
+                    if ($subscribemessage = forum_post_subscription($discussion)) {
+                        $timemessage = 4;
+                    }
+
+                    redirect(forum_go_back_to("view.php?f=$discussion->forum"), $message.$subscribemessage, $timemessage);
 
-                if ($subscribemessage = forum_post_subscription($discussion)) {
-                    $timemessage = 4;
+                } else {
+                    error(get_string("couldnotadd", "forum"), $errordestination);
                 }
-
-                redirect(forum_go_back_to("view.php?f=$post->forum"), $message.$subscribemessage, $timemessage);
 
-            } else {
-                error(get_string("couldnotadd", "forum"), $errordestination);
+                exit;
             }
-            exit;
         }
     }
 
-    if ($usehtmleditor = can_use_richtext_editor()) {
+    if ($usehtmleditor = can_use_html_editor()) {
         $defaultformat = FORMAT_HTML;
     } else {
         $defaultformat = FORMAT_MOODLE;
     }
 
-    if (isset($post->error)) {     // User is re-editing a failed posting
+    if (!empty($post->error)) {
+/// User is re-editing a failed posting
 
         // Set up all the required objects again, and reuse the same $post
 
@@ -202,11 +351,32 @@
             if (! $discussion = get_record("forum_discussions", "id", $post->discussion)) {
                 error("This post is not part of a discussion! ($post->discussion)");
             }
+        } else {
+            $discussion = new stdClass();
+            $newstopic = false;
+            if ($forum->type == 'news' && !$post->parent) {
+                $newstopic = true;
+            }
+            if ($newstopic && empty($post->timestartdisabled)) {
+                $discussion->timestart = make_timestamp($post->timestartyear, $post->timestartmonth, $post->timestartday);
+            } else {
+                $discussion->timestart = 0;
+            }
+            if ($newstopic && empty($post->timeenddisabled)) {
+                $discussion->timeend = make_timestamp($post->timeendyear, $post->timeendmonth, $post->timeendday);
+            } else {
+                $discussion->timeend = 0;
+            }
         }
 
-    } else if (isset($forum)) {      // User is starting a new discussion in a forum
+    } else if (!empty($forum)) {
+/// User is starting a new discussion in a forum
 
-        $SESSION->fromurl = $_SERVER["HTTP_REFERER"];
+        if (!empty($_SERVER["HTTP_REFERER"])) {
+            $SESSION->fromurl = $_SERVER["HTTP_REFERER"];
+        } else {
+            $SESSION->fromurl = '';
+        }
 
         if (! $forum = get_record("forum", "id", $forum)) {
             error("The forum number was incorrect ($forum)");
@@ -236,14 +406,10 @@
         $post->message = "";
         $post->format = $defaultformat;
 
-        $post->groupid = get_current_group($course->id);
-        if (isteacheredit($course->id) and $post->groupid == 0) {
-            $post->groupid = -1;
-        }
-
         forum_set_return();
 
-    } else if (!empty($reply)) {      // User is writing a new reply
+    } else if (!empty($reply)) {
+/// User is writing a new reply
 
         if (! $parent = forum_get_post_full($reply)) {
             error("Parent post ID was incorrect");
@@ -265,7 +431,7 @@
         if ($cm = get_coursemodule_from_instance("forum", $forum->id, $course->id)) {
             if (groupmode($course, $cm) and !isteacheredit($course->id)) {   // Make sure user can post here
                 $mygroupid = mygroupid($course->id);
-                if (!((empty($mygroupid) and $discussion->groupid == -1) || ($mygroupid == $discussion->groupid))) {
+                if (!((empty($mygroupid) and $discussion->groupid == -1) || (ismember($discussion->groupid)/*$mygroupid == $discussion->groupid*/))) {
                     error("Sorry, but you can not post in this discussion.");
                 }
             }
@@ -292,7 +458,8 @@
 
         unset($SESSION->fromdiscussion);
 
-    } else if (isset($edit)) {  // User is editing their own post
+    } else if (!empty($edit)) {
+/// User is editing their own post
 
         $adminedit = (isadmin() and !empty($CFG->admineditalways));
 
@@ -302,9 +469,6 @@
         if (($post->userid <> $USER->id) and !$adminedit) {
             error("You can't edit other people's posts!");
         }
-        if (((time() - $post->created) > $CFG->maxeditingtime) and !$adminedit) {
-            error( get_string("maxtimehaspassed", "forum", format_time($CFG->maxeditingtime)) );
-        }
         if ($post->parent) {
             if (! $parent = forum_get_post_full($post->parent)) {
                 error("Parent post ID was incorrect ($post->parent)");
@@ -316,6 +480,12 @@
         if (! $forum = get_record("forum", "id", $discussion->forum)) {
             error("The forum number was incorrect ($discussion->forum)");
         }
+        if (!($forum->type == 'news' && !$post->parent && $discussion->timestart > time())) {
+            if (((time() - $post->created) > $CFG->maxeditingtime) and !$adminedit) {
+                error( get_string("maxtimehaspassed", "forum", format_time($CFG->maxeditingtime)),
+                       "$CFG->wwwroot/mod/forum/discuss.php?d=$discussion->id#$post->id" );
+            }
+        }
         if (! $course = get_record("course", "id", $discussion->course)) {
             error("The course number was incorrect ($discussion->course)");
         }
@@ -330,7 +500,8 @@
         unset($SESSION->fromdiscussion);
 
 
-    } else if (isset($delete)) {  // User is deleting a post
+    } else if (!empty($delete)) {
+/// User is deleting a post
 
         if (! $post = forum_get_post_full($delete)) {
             error("Post ID was incorrect");
@@ -354,7 +525,7 @@
 
         $replycount = forum_count_replies($post);
 
-        if (isset($confirm)) {    // User has confirmed the delete
+        if (!empty($confirm) and confirm_sesskey()) {    // User has confirmed the delete
 
             if ($post->totalscore) {
                 notice(get_string("couldnotdeleteratings", "forum"),
@@ -394,7 +565,8 @@
             }
 
 
-        } else { // User just asked to delete something
+        } else {
+// User just asked to delete something
 
             forum_set_return();
 
@@ -405,7 +577,7 @@
                 }
                 print_header();
                 notice_yesno(get_string("deletesureplural", "forum", $replycount+1),
-                             "post.php?delete=$delete&amp;confirm=$delete",
+                             "post.php?delete=$delete&amp;confirm=$delete&amp;sesskey=".sesskey(),
                              $_SERVER["HTTP_REFERER"]);
 
                 forum_print_post($post, $course->id, $ownpost=false, $reply=false, $link=false);
@@ -420,7 +592,7 @@
             } else {
                 print_header();
                 notice_yesno(get_string("deletesure", "forum", $replycount),
-                             "post.php?delete=$delete&amp;confirm=$delete",
+                             "post.php?delete=$delete&amp;confirm=$delete&amp;sesskey=".sesskey(),
                              $_SERVER["HTTP_REFERER"]);
                 forum_print_post($post, $forum->course, $ownpost=false, $reply=false, $link=false);
             }
@@ -430,7 +602,8 @@
         die;
 
 
-    } else if (isset($prune)) {  // Teacher is pruning
+    } else if (!empty($prune)) {
+// Teacher is pruning
 
         if (!$post = forum_get_post_full($prune)) {
             error("Post ID was incorrect");
@@ -451,16 +624,19 @@
             $cm->id = 0;
         }
 
-        if (isset($_GET['name'])) {    // User has confirmed the prune
+        if (!empty($name) and confirm_sesskey()) {    // User has confirmed the prune
 
+            $newdiscussion = new object;
             $newdiscussion->course = $discussion->course;
             $newdiscussion->forum = $discussion->forum;
-            $newdiscussion->name = $name;
+            $newdiscussion->name = strip_tags($name, '<lang><span>'); // Strip all tags except multilang
             $newdiscussion->firstpost = $post->id;
             $newdiscussion->userid = $discussion->userid;
             $newdiscussion->groupid = $discussion->groupid;
             $newdiscussion->assessed = $discussion->assessed;
             $newdiscussion->usermodified = $post->userid;
+            $newdiscussion->timestart = $discussion->timestart;
+            $newdiscussion->timeend = $discussion->timeend;
 
             if (!$newid = insert_record('forum_discussions', $newdiscussion)) {
                 error('Could not create new discussion');
@@ -468,7 +644,7 @@
 
             $newpost->id = $post->id;
             $newpost->parent = 0;
-            $newpost->subject = $name;
+            $newpost->subject = $newdiscussion->name;
 
             if (!update_record("forum_posts", $newpost)) {
                 error('Could not update the original post');
@@ -572,6 +748,20 @@
 
     }
 
+// checkup
+    if (!empty($parent) && !forum_user_can_see_post($forum,$discussion,$post)) {
+        error("You cannot reply to this post");
+    }
+    if (empty($parent) && !forum_user_can_post_discussion($forum, false, '', $edit)) {
+        error("You cannot start a new discussion in this forum");
+    }
+
+    if ($forum->type == 'qanda' && !isteacher($forum->course) && !forum_user_has_posted($forum->id,$discussion->id,$USER->id)) {
+        notify(get_string('qandanotify','forum'));
+    }
+
+    forum_check_throttling($forum);
+
     if (!empty($parent)) {
         forum_print_post($parent, $course->id, $ownpost=false, $reply=false, $link=false);
         if (empty($post->edit)) {
@@ -580,7 +770,9 @@
             } else {
                 $user_read_array = array();
             }
-            forum_print_posts_threaded($parent->id, $course->id, 0, false, false, $user_read_array, $discussion->forum);
+            if ($forum->type != 'qanda' || forum_user_can_see_discussion($forum,$discussion)) {
+                forum_print_posts_threaded($parent->id, $course->id, 0, false, false, $user_read_array, $discussion->forum);
+            }
         }
         print_heading(get_string("yourreply", "forum").':');
     } else {
@@ -588,7 +780,11 @@
         if (!empty($forum->intro)) {
             print_simple_box(format_text($forum->intro), 'center');
         }
-        print_heading(get_string('yournewtopic', 'forum'));
+        if ($forum->type == 'qanda') {
+            print_heading(get_string('yournewquestion','forum'));
+        } else {
+            print_heading(get_string('yournewtopic', 'forum'));
+        }
     }
     echo '<center>';
     if (!empty($post->error)) {
@@ -596,6 +792,18 @@
     }
     echo '</center>';
 
+    if ($USER->id != $post->userid) {   // Not the original author, so add a message to the end
+        $data->date = userdate($post->modified);
+        if ($post->format == FORMAT_HTML) {
+            $data->name = '<a href="'.$CFG->wwwroot.'/user/view.php?id='.$USER->id.'&course='.$post->course.'">'.
+                           fullname($USER).'</a>';
+            $post->message .= '<p>(<span class="edited">'.get_string('editedby', 'forum', $data).'</span>)</p>';
+        } else {
+            $data->name = fullname($USER);
+            $post->message .= "\n\n(".get_string('editedby', 'forum', $data).')';
+        }
+    }
+
     print_simple_box_start("center");
     require("post.html");
     print_simple_box_end();
