@@ -1,4 +1,4 @@
-<?php  // $Id: view.php,v 1.67.2.4 2005/07/06 00:35:37 defacer Exp $
+<?php  // $Id: view.php,v 1.88.2.5 2006/08/10 15:31:00 skodak Exp $
 
 // This page prints a particular instance of quiz
 
@@ -7,13 +7,12 @@
     require_once($CFG->libdir.'/blocklib.php');
     require_once('pagelib.php');
 
-    $id          = optional_param('id', 0, PARAM_INT); // Course Module ID, or
-    $q           = optional_param('q',  0, PARAM_INT);  // quiz ID
-    $edit        = optional_param('edit', '');
-
-
+    $id   = optional_param('id', 0, PARAM_INT); // Course Module ID, or
+    $q    = optional_param('q',  0, PARAM_INT);  // quiz ID
+    $edit = optional_param('edit', -1, PARAM_BOOL);
+    
     if ($id) {
-        if (! $cm = get_record("course_modules", "id", $id)) {
+        if (! $cm = get_coursemodule_from_id('quiz', $id)) {
             error("There is no coursemodule with id $id");
         }
 
@@ -57,15 +56,14 @@
 
 // Print the page header
 
-    if (!empty($edit) && $PAGE->user_allowed_editing()) {
-        if ($edit == 'on') {
-            $USER->editing = true;
-        } else if ($edit == 'off') {
-            $USER->editing = false;
-        }
+    if (($edit != -1) and $PAGE->user_allowed_editing()) {
+        $USER->editing = $edit;
     }
 
-    $PAGE->print_header($course->shortname.': %fullname%');
+    //only check pop ups if the user is not a teacher, and popup is set
+    
+    $bodytags = (isteacher($course->id) or !$quiz->popup)?'':'onload="popupchecker(\'This section of the test is in secure mode, this means that you need to take the quiz in a secure window. Please turn off your popup blocker. Thank you.\');"';
+    $PAGE->print_header($course->shortname.': %fullname%','',$bodytags);
 
     echo '<table id="layout-table"><tr>';
 
@@ -77,7 +75,7 @@
 
     echo '<td id="middle-column">';
 
-    $available = ($quiz->timeopen < $timenow and $timenow < $quiz->timeclose) || $isteacher;
+    $available = ($quiz->timeopen < $timenow and ($timenow < $quiz->timeclose or !$quiz->timeclose)) || $isteacher;
 
 // Print the main part of the page
 
@@ -89,7 +87,8 @@
     print_heading(format_string($quiz->name));
 
     if (trim(strip_tags($quiz->intro))) {
-        print_simple_box(format_text($quiz->intro), "center");
+        $formatoptions->noclean = true;
+        print_simple_box(format_text($quiz->intro, FORMAT_MOODLE, $formatoptions), "center");
     }
 
     if ($quiz->attempts > 1) {
@@ -102,7 +101,7 @@
         if ($quiz->timelimit) {
             echo "<p align=\"center\">".get_string("quiztimelimit","quiz", format_time($quiz->timelimit * 60))."</p>";
         }
-        echo "<p align=\"center\">".get_string("quizavailable", "quiz", userdate($quiz->timeclose));
+        quiz_view_dates($quiz);
     } else if ($timenow < $quiz->timeopen) {
         echo "<p align=\"center\">".get_string("quiznotavailable", "quiz", userdate($quiz->timeopen));
     } else {
@@ -112,15 +111,13 @@
 
     // This is all the teacher will get
     if ($isteacher) {
-        
-        if ($attemptcount = count_records('quiz_attempts', 'quiz', $quiz->id, 'preview', 0)) {
-
-            $strviewallanswers  = get_string("viewallanswers", "quiz", $attemptcount);
-            $usercount = count_records_select('quiz_attempts', "quiz = '$quiz->id' AND preview = '0'", 'COUNT(DISTINCT userid)');
-            $strusers  = $course->students;
+        if ($a->attemptnum = count_records('quiz_attempts', 'quiz', $quiz->id, 'preview', 0)) {
+            $a->studentnum = count_records_select('quiz_attempts', "quiz = '$quiz->id' AND preview = '0'", 'COUNT(DISTINCT userid)');
+            $a->studentstring  = $course->students;
     
-            notify("<a href=\"report.php?mode=overview&amp;id=$cm->id\">$strviewallanswers ($usercount $strusers)</a>");
+            notify("<a href=\"report.php?mode=overview&amp;id=$cm->id\">".get_string('numattempts', 'quiz', $a).'</a>');
         }
+        
         echo '</td></tr></table>';
         print_footer($course);
         exit;
@@ -130,10 +127,10 @@
 
         $wwwroot = $CFG->wwwroot.'/login/index.php';
         if (!empty($CFG->loginhttps)) {
-            $wwwroot = str_replace('http','https', $wwwroot);
+            $wwwroot = str_replace('http:','https:', $wwwroot);
         }
 
-        notice_yesno(get_string('guestno', 'quiz').'<br /><br />'.get_string('liketologin'),
+        notice_yesno(get_string('guestsno', 'quiz').'<br /><br />'.get_string('liketologin'),
                      $wwwroot, $_SERVER['HTTP_REFERER']);
         print_footer($course);
         echo '</td></tr></table>';
@@ -156,7 +153,7 @@
     $strtimetaken     = get_string("timetaken", "quiz");
     $strtimecompleted = get_string("timecompleted", "quiz");
     $strgrade         = get_string("grade");
-    $strmarks          = get_string('marks', 'quiz');
+    $strmarks         = get_string('marks', 'quiz');
     $strbestgrade     = $QUIZ_GRADE_METHOD[$quiz->grademethod];
 
     $windowoptions = "left=0, top=0, channelmode=yes, fullscreen=yes, scrollbars=yes, resizeable=no, directories=no, toolbar=no, titlebar=no, location=no, status=no, menubar=no";
@@ -164,25 +161,37 @@
     $mygrade = quiz_get_best_grade($quiz, $USER->id);
 
 /// Now print table with existing attempts
+    $gradecolumn=0;
+    $overallstats=1;
 
-    if ($numattempts) {
-
+    if ($attempts) {
+                    
+        //step thru each attempt, checking there are any attempts
+        //for which the score can be displayed (need grade columns),
+        //and checking if overall grades can be displayed - no attempts for 
+        //which the score cannot be displayed
+        foreach ($attempts as $attempt) {            
+            $attemptoptions = quiz_get_reviewoptions($quiz, $attempt, $isteacher);
+            $attemptoptions->scores ? $gradecolumn=1 : $overallstats=0;                    
+        }
     /// prepare table header
-        if ($quiz->grade and $quiz->sumgrades) { // Grades used so have more columns in table
+        $table->head = array($strattempt, $strtimecompleted);
+        $table->align = array("center", "left");
+        $table->size = array("", "");
+        if ($gradecolumn && $quiz->grade and $quiz->sumgrades) { // Grades used so have more columns in table
             if ($quiz->grade <> $quiz->sumgrades) {
-                $table->head = array($strattempt, $strtimetaken, $strtimecompleted, "$strmarks / $quiz->sumgrades", "$strgrade / $quiz->grade");
-                $table->align = array("center", "center", "left", "right", "right");
-                $table->size = array("", "", "", "", "");
-            } else {
-                $table->head = array($strattempt, $strtimetaken, $strtimecompleted, "$strgrade / $quiz->grade");
-                $table->align = array("center", "center", "left", "right");
-                $table->size = array("", "", "", "");
+                $table->head[] = "$strmarks / $quiz->sumgrades";
+                $table->align[] = 'right';
+                $table->size[] = '';
             }
-
-        } else {  // No grades are being used
-            $table->head = array($strattempt, $strtimetaken, $strtimecompleted);
-            $table->align = array("center", "center", "left");
-            $table->size = array("", "", "");
+            $table->head[] = "$strgrade / $quiz->grade";
+            $table->align[] = 'right';
+            $table->size[] = '';
+        }
+        if (isset($quiz->showtimetaken)) {
+            $table->head[] = $strtimetaken;
+            $table->align[] = 'center';
+            $table->size[] = '';
         }
 
     /// One row for each attempt
@@ -196,21 +205,21 @@
                 $datecompleted = userdate($attempt->timefinish);
             } else if ($available) { // The student can continue this attempt, so put appropriate link
                 $timetaken = format_time(time() - $attempt->timestart);
-                $strconfirmstartattempt = addslashes(get_string("confirmstartattempt","quiz"));
                 $datecompleted  = "\n".'<script language="javascript" type="text/javascript">';
                 $datecompleted .= "\n<!--\n"; // -->
+                if (!empty($CFG->usesid) && !isset($_COOKIE[session_name()])) {
+                    $attempturl=sid_process_url("attempt.php?id=$cm->id");
+                } else {
+                    $attempturl="attempt.php?id=$cm->id";
+                };
                 if (!empty($quiz->popup)) {
                     $datecompleted .= "var windowoptions = 'left=0, top=0, height='+window.screen.height+
-                     ', width='+window.screen.width+', channelmode=yes, fullscreen=yes, scrollbars=yes, '+
-                     'resizeable=no, directories=no, toolbar=no, titlebar=no, location=no, status=no, '+
-                     'menubar=no';\n";
-                    $jslink  = 'javascript:';
-                    if ($quiz->timelimit) {
-                        $jslink .=  "if (confirm('$strconfirmstartattempt')) ";
-                    }
-                    $jslink .= "var popup = window.open(\\'attempt.php?id=$cm->id\\', \\'quizpopup\\', windowoptions);";
+                            ', width='+window.screen.width+', channelmode=yes, fullscreen=yes, scrollbars=yes, '+
+                            'resizeable=no, directories=no, toolbar=no, titlebar=no, location=no, status=no, '+
+                            'menubar=no';\n";
+                    $jslink  = "javascript:var popup = window.open(\\'$attempturl\\', \\'quizpopup\\', windowoptions);";
                 } else {
-                    $jslink = "attempt.php?id=$cm->id";
+                    $jslink = $attempturl;
                 }
 
                 $linktext = get_string('continueattemptquiz', 'quiz');
@@ -222,16 +231,18 @@
                 $datecompleted .= '</noscript>';
             } else { // attempt was not completed but is also not available any more.
                 $timetaken = format_time($quiz->timeclose - $attempt->timestart);
-                $datecompleted = userdate($quiz->timeclose);
+                $datecompleted = $quiz->timeclose ? userdate($quiz->timeclose) : '';
             }
 
+            $attemptoptions = quiz_get_reviewoptions($quiz, $attempt, $isteacher);
         /// prepare strings for attempt number, mark and grade
-            if ($quiz->grade and $quiz->sumgrades) {
+            //if attempt's score is allowed to be viewed, & qz->sumgrades and qz->sumgrades defined:
+            if ($attemptoptions->scores && $quiz->grade and $quiz->sumgrades) {
                 $attemptmark  = round($attempt->sumgrades,$quiz->decimalpoints);
                 $attemptgrade = round(($attempt->sumgrades/$quiz->sumgrades)*$quiz->grade,$quiz->decimalpoints);
 
                 // highlight the highest grade if appropriate
-                if ($attemptgrade == $mygrade and ($quiz->grademethod == QUIZ_GRADEHIGHEST)) {
+                if ($overallstats && $attemptgrade == $mygrade and ($quiz->grademethod == QUIZ_GRADEHIGHEST)) {
                     $attemptgrade = "<span class=\"highlight\">$attemptgrade</span>";
                 }
 
@@ -251,12 +262,10 @@
 
                 if ($quiz->grade <> $quiz->sumgrades) {
                     $table->data[] = array( $attempt->attempt,
-                                            $timetaken,
                                             $datecompleted,
                                             $attemptmark, $attemptgrade);
                 } else {
                     $table->data[] = array( $attempt->attempt,
-                                            $timetaken,
                                             $datecompleted,
                                             $attemptgrade);
                 }
@@ -268,9 +277,20 @@
                         $attempt->attempt = "<a href=\"attempt.php?id=$id\">#$attempt->attempt</a>";
                     }
                 }
-                $table->data[] = array( $attempt->attempt,
-                                        $timetaken,
-                                        $datecompleted);
+
+                $helpbutton=helpbutton('missing\ grade', get_string('wheregrade', 'quiz'), 'quiz', true, false, '',true);
+                if($gradecolumn) {
+                    $table->data[] = array( $attempt->attempt,
+                                            $datecompleted,
+                                            $helpbutton);
+                     
+                } else {
+                    $table->data[] = array( $attempt->attempt,
+                                            $datecompleted);
+                }
+            }
+            if (isset($quiz->showtimetaken)) {
+                $table->data[] = $timetaken;
             }
         }
         print_table($table);
@@ -280,23 +300,52 @@
         print_heading(get_string("noquestions", "quiz"));
     } else {
         if ($numattempts < $quiz->attempts or !$quiz->attempts) {
+          
             if ($available) {
                 $options["id"] = $cm->id;
-                if ($numattempts and $quiz->grade) {
+                //if overall stats are allowed (no attemps' grade not visible),
+                //and there is at least one attempt, and quiz->grade:
+                if ($overallstats and $numattempts and $quiz->grade) {
                     print_heading("$strbestgrade: $mygrade / $quiz->grade.");
                 }
-                $strconfirmstartattempt = addslashes(get_string("confirmstartattempt","quiz"));
+                
                 echo "<br />";
                 echo "</p>";
                 echo "<div align=\"center\">";
-
-                include("view_js.php");
-
+                if ($quiz->delay1 or $quiz->delay2) {
+                     //quiz enforced time delay
+                     $lastattempt_obj = get_record_select('quiz_attempts', "quiz = $quiz->id AND attempt = $numattempts AND userid = $USER->id", 'timefinish');
+                     if ($lastattempt_obj) {
+                         $lastattempt = $lastattempt_obj->timefinish;
+                     }
+                     if($numattempts == 1 && $quiz->delay1) {
+                         if ($timenow - $quiz->delay1 > $lastattempt) {
+                              print_start_quiz_button($quiz, $attempts, $numattempts, $unfinished, $cm);
+                         } else {
+                             $notify_msg = get_string('temporaryblocked', 'quiz') . '<b>'. userdate($lastattempt + $quiz->delay1). '<b>';
+                             print_simple_box($notify_msg, "center");
+                         }
+                     } else if($numattempts > 1 && $quiz->delay2) {
+                         if ($timenow - $quiz->delay2 > $lastattempt) {
+                              print_start_quiz_button($quiz, $attempts, $numattempts, $unfinished, $cm);
+                         } else {
+                              $notify_msg = get_string('temporaryblocked', 'quiz') . '<b>'. userdate($lastattempt + $quiz->delay2). '<b>';
+                              print_simple_box($notify_msg, "center");
+                         }
+                     } else {
+                         print_start_quiz_button($quiz, $attempts, $numattempts, $unfinished, $cm);
+                     }
+                } else {
+                     print_start_quiz_button($quiz, $attempts, $numattempts, $unfinished, $cm);
+                }     
                 echo "</div>\n";
             }
         } else {
             print_heading(get_string("nomoreattempts", "quiz"));
-            if ($quiz->grade) {
+            //if $quiz->grade and $quiz->sumgrades, and student is allowed to 
+            //see summary statistics (no attempt's grade is concealed),
+            //show the student their final grade
+            if ($quiz->grade and $quiz->sumgrades and $overallstats) {
                 print_heading(get_string("yourfinalgradeis", "quiz", "$mygrade / $quiz->grade"));
             }
             print_continue('../../course/view.php?id='.$course->id);
@@ -313,13 +362,66 @@
         if (!($quiz->review & QUIZ_REVIEW_RESPONSES)) {
             return false;
         }
-        if ((time() < $quiz->timeclose) and !($quiz->review & QUIZ_REVIEW_OPEN)) {
+        if ((!$quiz->timeclose or time() < $quiz->timeclose) and !($quiz->review & QUIZ_REVIEW_OPEN)) {
             return false;
         }
-        if ((time() > $quiz->timeclose) and !($quiz->review & QUIZ_REVIEW_CLOSED)) {
+        if (($quiz->timeclose and time() > $quiz->timeclose) and !($quiz->review & QUIZ_REVIEW_CLOSED)) {
             return false;
         }
         return true;
     }
+    
+    
+    function print_start_quiz_button($quiz, $attempts, $numattempts, $unfinished, $cm) {
+        $strconfirmstartattempt =  '';
+        
+        if ($unfinished) {
+            $buttontext = get_string('continueattemptquiz', 'quiz');
+        } else {
+            if ($numattempts) {
+                $buttontext = get_string('reattemptquiz', 'quiz');
+            } else {
+                $buttontext = get_string('attemptquiznow', 'quiz');
+            }
+            if ($quiz->timelimit && $quiz->attempts) {
+                $strconfirmstartattempt = addslashes(get_string('confirmstartattempttimelimit','quiz', $quiz->attempts));
+            } else if ($quiz->timelimit) {
+                $strconfirmstartattempt = addslashes(get_string('confirmstarttimelimit','quiz'));
+            } else if ($quiz->attempts) {
+                $strconfirmstartattempt = addslashes(get_string('confirmstartattemptlimit','quiz', $quiz->attempts));
+            }
+        }
+        $buttontext = htmlspecialchars($buttontext, ENT_QUOTES);
+
+        if (!empty($quiz->popup)) {
+            $window = 'quizpopup';
+            $windowoptions = "left=0, top=0, height='+window.screen.height+', " .
+                    "width='+window.screen.width+', channelmode=yes, fullscreen=yes, " .
+                    "scrollbars=yes, resizeable=no, directories=no, toolbar=no, " .
+                    "titlebar=no, location=no, status=no, menubar=no";
+        } else {
+            $window = '_self';
+            $windowoptions = '';
+        }
+
+        $attempturl = "attempt.php?id=$cm->id";
+        if (!empty($CFG->usesid) && !isset($_COOKIE[session_name()])) {
+            $attempturl = sid_process_url($attempturl);
+        }
+?>
+<script language="javascript" type="text/javascript">
+<!--
+document.write('<input type="button" value="<?php echo $buttontext ?>" onclick="javascript: <?php 
+        if ($strconfirmstartattempt) {
+            echo "if (confirm(\\'".addslashes($strconfirmstartattempt)."\\'))"; 
+        } 
+?> window.open(\'<?php echo $attempturl ?>\', \'<?php echo $window ?>\', \'<?php echo $windowoptions ?>\'); " />');
+// -->
+</script>
+<noscript>
+    <strong><?php print_string('noscript', 'quiz'); ?></strong>
+</noscript>
+<?php
+    }
 
 ?>
